@@ -8,7 +8,7 @@ import socket
 import subprocess
 import sys
 from liblo import Server, Address
-from PyQt5.QtCore import QLocale, QTranslator, QT_VERSION_STR, QFile
+from PyQt5.QtCore import QT_VERSION_STR, QFile
 from PyQt5.QtGui import QIcon, QPalette
 
 # get qt version in list of ints
@@ -23,7 +23,7 @@ if QT_VERSION < (5, 6):
         "WARNING: You are using a version of QT older than 5.6.\n"
         + "You won't be warned if a process can't be launch.\n")
 
-VERSION = "0.9.2"
+VERSION = "0.10.1"
 
 APP_TITLE = 'RaySession'
 DEFAULT_SESSION_ROOT = "%s/Ray Sessions" % os.getenv('HOME')
@@ -31,6 +31,7 @@ SCRIPTS_DIR = 'ray-scripts'
 NOTES_PATH = 'ray-notes'
 factory_session_templates = (
     'with_jack_patch', 'with_jack_config', 'scripted')
+RAYNET_BIN = 'ray-network'
 
 class PrefixMode:
     CUSTOM = 0
@@ -39,45 +40,45 @@ class PrefixMode:
 
 
 class ClientStatus:
-    STOPPED =  0
-    LAUNCH  =  1
-    OPEN    =  2
-    READY   =  3
-    PRECOPY =  4
-    COPY    =  5
-    SAVE    =  6
-    SWITCH  =  7
-    QUIT    =  8
-    NOOP    =  9
-    ERROR   = 10
+    STOPPED = 0
+    LAUNCH = 1
+    OPEN = 2
+    READY = 3
+    PRECOPY = 4
+    COPY = 5
+    SAVE = 6
+    SWITCH = 7
+    QUIT = 8
+    NOOP = 9
+    ERROR = 10
     REMOVED = 11
-    UNDEF   = 12
-    SCRIPT  = 13
+    UNDEF = 12
+    SCRIPT = 13
 
 
 class ServerStatus:
-    OFF      =  0
-    NEW      =  1
-    OPEN     =  2
-    CLEAR    =  3
-    SWITCH   =  4
-    LAUNCH   =  5
-    PRECOPY  =  6
-    COPY     =  7
-    READY    =  8
-    SAVE     =  9
-    CLOSE    = 10
+    OFF = 0
+    NEW = 1
+    OPEN = 2
+    CLEAR = 3
+    SWITCH = 4
+    LAUNCH = 5
+    PRECOPY = 6
+    COPY = 7
+    READY = 8
+    SAVE = 9
+    CLOSE = 10
     SNAPSHOT = 11
-    REWIND   = 12
+    REWIND = 12
     WAIT_USER = 13
-    OUT_SAVE  = 14
+    OUT_SAVE = 14
     OUT_SNAPSHOT = 15
     SCRIPT = 16
 
 
 class NSMMode:
-    NO_NSM  = 0
-    CHILD   = 1
+    NO_NSM = 0
+    CHILD = 1
     NETWORK = 2
 
 
@@ -96,6 +97,8 @@ class Option:
     HAS_GIT = 0x020
     SNAPSHOTS = 0x040
     SESSION_SCRIPTS = 0x080
+    GUI_STATES = 0x100
+
 
 class Err:
     OK = 0
@@ -178,9 +181,9 @@ def ifDebug(string):
     if debug:
         sys.stderr.write("%s\n" % string)
 
-def setDebug(bool):
+def setDebug(bool_debug: bool):
     global debug
-    debug = bool
+    debug = bool_debug
 
 def versionToTuple(version_str):
     version_list = []
@@ -276,8 +279,19 @@ def isGitTaggable(string):
 def highlightText(string):
     if "'" in string:
         return '"%s"' % string
-    else:
-        return "'%s'" % string
+    return "'%s'" % string
+
+def isValidFullPath(path: str)->bool:
+    if not path.startswith('/'):
+        return False
+
+    for forbidden in ('//', '/./', '/../'):
+        if forbidden in path:
+            return False
+
+    if path.endswith(('/.', '/..')):
+        return False
+    return True
 
 def isOscPortFree(port):
     try:
@@ -310,7 +324,6 @@ def getFreeOscPort(default=16187):
     del testport
     return daemon_port
 
-
 def isValidOscUrl(url):
     try:
         address = liblo.Address(url)
@@ -318,7 +331,6 @@ def isValidOscUrl(url):
     except BaseException:
         return False
 
-
 def getLibloAddress(url):
     valid_url = False
     try:
@@ -464,16 +476,14 @@ def getThis192():
     except BaseException:
         return ''
 
-
 def getMachine192(hostname=None):
     if hostname is None:
         return getThis192()
-    else:
-        if hostname in ('localhost', socket.gethostname()):
-            return getThis192()
 
-        return socket.gethostbyname(hostname)
+    if hostname in ('localhost', socket.gethostname()):
+        return getThis192()
 
+    return socket.gethostbyname(hostname)
 
 def getMachine192ByUrl(url):
     try:
@@ -486,7 +496,6 @@ def getMachine192ByUrl(url):
 
     return getMachine192(hostname)
 
-
 def getNetUrl(port):
     try:
         ips = subprocess.check_output(['hostname', '-I']).decode()
@@ -499,7 +508,6 @@ def getNetUrl(port):
 
     return "osc.udp://%s:%i/" % (ip, port)
 
-
 def shellLineToArgs(string):
     try:
         args = shlex.split(string)
@@ -510,11 +518,10 @@ def shellLineToArgs(string):
 
 def areTheyAllString(args):
     for arg in args:
-        if type(arg) != str:
+        if not isinstance(arg, str):
             return False
     return True
 
-
 def getAppIcon(icon_name, widget):
     dark = bool(
         widget.palette().brush(
@@ -535,6 +542,16 @@ def getAppIcon(icon_name, widget):
                 icon = QIcon()
                 icon.addFile(filename)
                 break
+            
+    if icon.isNull():
+        for path in ('/usr/local', '/usr', '%s/.local' % os.getenv('HOME')):
+            for ext in ('png', 'svg', 'svgz', 'xpm'):
+                filename = "%s/share/pixmaps/%s.%s" % (path, icon_name, ext)
+                if QFile.exists(filename):
+                    del icon
+                    icon = QIcon()
+                    icon.addFile(filename)
+                    break
 
     return icon
 
@@ -558,13 +575,20 @@ def getFullPath(root, session_name):
 
     return spath
 
-def protocolToStr(protocol:int)->str:
+def protocolToStr(protocol: int)->str:
     if protocol == Protocol.RAY_HACK:
         return "Ray-Hack"
-    elif protocol == Protocol.RAY_NET:
-        return "Net-Session"
+    if protocol == Protocol.RAY_NET:
+        return "Ray-Net"
     return "NSM"
 
+def protocolFromStr(protocol_str: str)->int:
+    if protocol_str.lower() in ('ray_hack', 'ray-hack'):
+        return Protocol.RAY_HACK
+    elif protocol_str.lower() in ('ray_net', 'ray-net'):
+        return Protocol.RAY_NET
+    return Protocol.NSM
+
 class ClientData:
     client_id = ''
     protocol = Protocol.NSM
@@ -581,8 +605,10 @@ class ClientData:
     capabilities = ''
     check_last_save = True
     ignored_extensions = getGitIgnoredExtensions()
-    useless_str = ''
+    template_origin = ''
     useless_int = 0
+    ray_hack = None
+    ray_net = None
 
     @staticmethod
     def sisi():
@@ -603,12 +629,18 @@ class ClientData:
                 client.icon,
                 client.capabilities, int(client.check_last_save),
                 client.ignored_extensions,
-                client.useless_str, client.useless_int)
+                client.template_origin, client.useless_int)
 
     def gui_init(self, client_id, protocol):
         self.client_id = client_id
         self.protocol = protocol
 
+    def set_ray_hack(self, ray_hack):
+        self.ray_hack = ray_hack
+
+    def set_ray_net(self, ray_net):
+        self.ray_net = ray_net
+
     def update(self, client_id, protocol,
                executable, arguments, pre_env,
                name, prefix_mode, custom_prefix,
@@ -616,7 +648,7 @@ class ClientData:
                icon,
                capabilities, check_last_save,
                ignored_extensions,
-               useless_str, useless_int,
+               template_origin, useless_int,
                secure=False):
         self.executable_path = str(executable)
         self.arguments = str(arguments)
@@ -629,6 +661,7 @@ class ClientData:
 
         self.check_last_save = bool(check_last_save)
         self.ignored_extensions = str(ignored_extensions)
+        self.template_origin = template_origin
 
         if secure:
             return
@@ -654,6 +687,15 @@ class ClientData:
 
     def spread(self)->tuple:
         return ClientData.spreadClient(self)
+    
+    def prettier_name(self)->str:
+        if self.label:
+            return self.label
+        if (self.protocol != Protocol.RAY_HACK
+                and self.name):
+            return self.name
+        return self.executable_path
+
 
 class RayHack():
     config_file = ""
@@ -698,3 +740,29 @@ class RayHack():
         return (self.config_file, self.save_sig, self.stop_sig,
                 int(self.wait_win), self.no_save_level,
                 self.useless_str, self.useless_int)
+
+class RayNet():
+    daemon_url = ''
+    session_root = ''
+    session_template = ''
+    duplicate_state = -1
+    running_daemon_url = ''
+    running_session_root =''
+
+    @staticmethod
+    def sisi():
+        return 'sss'
+
+    @staticmethod
+    def newFrom(*args):
+        ray_net = RayNet()
+        ray_net.update(*args)
+        return ray_net
+
+    def update(self, daemon_url, session_root, session_template):
+        self.daemon_url = daemon_url
+        self.session_root = session_root
+        self.session_template = session_template
+
+    def spread(self)->tuple:
+        return (self.daemon_url, self.session_root, self.session_template)
