@@ -39,7 +39,10 @@ class Client(ServerSender, ray.ClientData):
     _reply_message = None
 
     #can be directly changed by OSC thread
-    gui_visible = True
+    gui_visible = False
+    gui_has_been_visible = False
+    show_gui_ordered = False
+    dirty = 0
     progress = 0
 
     #have to be modified by main thread for security
@@ -62,11 +65,6 @@ class Client(ServerSender, ray.ClientData):
     sent_to_gui = False
     switch_state = ray.SwitchState.NONE
 
-    net_session_template = ''
-    net_session_root = ''
-    net_daemon_url = ''
-    net_duplicate_state = -1
-
     ignored_extensions = ray.getGitIgnoredExtensions()
 
     last_save_time = 0.00
@@ -125,6 +123,7 @@ class Client(ServerSender, ray.ClientData):
         self._open_timer.timeout.connect(self.openTimerTimeout)
 
         self.ray_hack = ray.RayHack()
+        self.ray_net = ray.RayNet()
         self.scripter = ClientScripter(self)
 
         self.ray_hack_waiting_win = False
@@ -138,6 +137,11 @@ class Client(ServerSender, ray.ClientData):
 
         self.send(self.addr, *args)
 
+    def message(self, message: str):
+        if self.session is None:
+            return
+        self.session.message(message)
+
     def sendReplyToCaller(self, slot, message):
         src_addr, src_path = self._osc_srcs[slot]
         if src_addr:
@@ -189,6 +193,7 @@ class Client(ServerSender, ray.ClientData):
         self.auto_start = bool(ctx.attribute('launched') != '0')
         self.check_last_save = bool(ctx.attribute('check_last_save') != '0')
         self.start_gui_hidden = bool(ctx.attribute('gui_visible') == '0')
+        self.template_origin = ctx.attribute('template_origin')
         self._from_nsm_file = bool(ctx.attribute('from_nsm_file') == '1')
 
         self.updateInfosFromDesktopFile()
@@ -207,6 +212,10 @@ class Client(ServerSender, ray.ClientData):
             if ext and not ext in global_exts:
                 self.ignored_extensions += " %s" % ext
 
+        open_duration = ctx.attribute('last_open_duration')
+        if open_duration.replace('.', '', 1).isdigit():
+            self.last_open_duration = float(open_duration)
+
         prefix_mode = ctx.attribute('prefix_mode')
 
         if (prefix_mode and prefix_mode.isdigit()
@@ -215,11 +224,7 @@ class Client(ServerSender, ray.ClientData):
             if self.prefix_mode == ray.PrefixMode.CUSTOM:
                 self.custom_prefix = ctx.attribute('custom_prefix')
 
-        protocol = ctx.attribute('protocol')
-        if protocol.lower() in ('ray_hack', 'ray-hack'):
-            self.protocol = ray.Protocol.RAY_HACK
-        elif protocol.lower() in ('ray_net', 'ray-net'):
-            self.protocol = ray.Protocol.RAY_NET
+        self.protocol = ray.protocolFromStr(ctx.attribute('protocol'))
 
         if self.protocol == ray.Protocol.RAY_HACK:
             self.ray_hack.config_file = ctx.attribute('config_file')
@@ -237,16 +242,13 @@ class Client(ServerSender, ray.ClientData):
             if no_save_level.isdigit() and 0 <= int(no_save_level) <= 2:
                 self.ray_hack.no_save_level = int(no_save_level)
 
-        self.net_session_template = ctx.attribute('net_session_template')
+        # backward compatibility with network session
+        if (self.protocol == ray.Protocol.NSM
+                and basename(self.executable_path) == 'ray-network'):
+            self.protocol = ray.Protocol.RAY_NET
 
-        open_duration = ctx.attribute('last_open_duration')
-        if open_duration.replace('.', '', 1).isdigit():
-            self.last_open_duration = float(open_duration)
-
-        if basename(self.executable_path) == 'ray-network':
             if self.arguments:
-                eat_url = False
-                eat_root = False
+                eat_url = eat_root = False
 
                 for arg in shlex.split(self.arguments):
                     if arg in ('--daemon-url', '-u'):
@@ -261,11 +263,23 @@ class Client(ServerSender, ray.ClientData):
                         continue
 
                     if eat_url:
-                        self.net_daemon_url = arg
+                        self.ray_net.daemon_url = arg
                         eat_url = False
                     elif eat_root:
-                        self.net_session_root = arg
+                        self.ray_net.session_root = arg
                         eat_root = False
+            self.ray_net.session_template = ctx.attribute('net_session_template')
+
+        elif self.protocol == ray.Protocol.RAY_NET:
+            self.ray_net.daemon_url = ctx.attribute('net_daemon_url')
+            self.ray_net.session_root = ctx.attribute('net_session_root')
+            self.ray_net.session_template = ctx.attribute('net_session_template')
+
+        if self.protocol == ray.Protocol.RAY_NET:
+            # neeeded only to know if RAY_NET client is capable of switch
+            self.executable_path = ray.RAYNET_BIN
+            if self.ray_net.daemon_url and self.ray_net.session_root:
+                self.arguments = self.getRayNetArgumentsLine()
 
         if ctx.attribute('id'):
             #session use "id" for absolutely needed client_id
@@ -288,7 +302,11 @@ class Client(ServerSender, ray.ClientData):
                     self.custom_data[attribute_str] = value
 
     def writeXmlProperties(self, ctx):
-        ctx.setAttribute('executable', self.executable_path)
+        if self.protocol != ray.Protocol.RAY_NET:
+            ctx.setAttribute('executable', self.executable_path)
+            if self.arguments:
+                ctx.setAttribute('arguments', self.arguments)
+
         ctx.setAttribute('name', self.name)
         if self.desktop_file:
             ctx.setAttribute('desktop_file', self.desktop_file)
@@ -300,36 +318,38 @@ class Client(ServerSender, ray.ClientData):
             ctx.setAttribute('icon', self.icon)
         if not self.check_last_save:
             ctx.setAttribute('check_last_save', 0)
-        if self.arguments:
-            ctx.setAttribute('arguments', self.arguments)
 
         if self.prefix_mode != ray.PrefixMode.SESSION_NAME:
             ctx.setAttribute('prefix_mode', self.prefix_mode)
-
             if self.prefix_mode == ray.PrefixMode.CUSTOM:
                 ctx.setAttribute('custom_prefix', self.custom_prefix)
 
         if self.isCapableOf(':optional-gui:'):
-            if self.executable_path != 'ray-proxy':
-                if self.start_gui_hidden:
-                    ctx.setAttribute('gui_visible', '0')
+            ctx.setAttribute('gui_visible',
+                             str(int(not self.start_gui_hidden)))
 
         if self._from_nsm_file:
             ctx.setAttribute('from_nsm_file', 1)
 
+        if self.template_origin:
+            ctx.setAttribute('template_origin', self.template_origin)
+
         if self.protocol != ray.Protocol.NSM:
             ctx.setAttribute('protocol', ray.protocolToStr(self.protocol))
 
-        if self.isRayHack():
-            ctx.setAttribute('config_file', self.ray_hack.config_file)
-            ctx.setAttribute('save_signal', self.ray_hack.save_sig)
-            ctx.setAttribute('stop_signal', self.ray_hack.stop_sig)
-            ctx.setAttribute('wait_win', int(self.ray_hack.wait_win))
-            ctx.setAttribute('no_save_level', self.ray_hack.no_save_level)
+            if self.protocol == ray.Protocol.RAY_HACK:
+                ctx.setAttribute('config_file', self.ray_hack.config_file)
+                ctx.setAttribute('save_signal', self.ray_hack.save_sig)
+                ctx.setAttribute('stop_signal', self.ray_hack.stop_sig)
+                ctx.setAttribute('wait_win', int(self.ray_hack.wait_win))
+                ctx.setAttribute('no_save_level', self.ray_hack.no_save_level)
 
-        if self.net_session_template:
-            ctx.setAttribute('net_session_template',
-                             self.net_session_template)
+            elif self.protocol == ray.Protocol.RAY_NET:
+                ctx.setAttribute('net_daemon_url', self.ray_net.daemon_url)
+                ctx.setAttribute('net_session_root',
+                                 self.ray_net.session_root)
+                ctx.setAttribute('net_session_template',
+                                 self.ray_net.session_template)
 
         if self.ignored_extensions != ray.getGitIgnoredExtensions():
             ignored = ""
@@ -373,7 +393,7 @@ class Client(ServerSender, ray.ClientData):
         self._reply_errcode = errcode
 
         if self._reply_errcode:
-            Terminal.message("Client \"%s\" replied with error: %s (%i)"
+            self.message("Client \"%s\" replied with error: %s (%i)"
                                 % (self.name, message, errcode))
 
             if self.pending_command == ray.Command.SAVE:
@@ -405,6 +425,14 @@ class Client(ServerSender, ray.ClientData):
                                         time.time() - self._last_announce_time
                 self.sendReplyToCaller(OSC_SRC_OPEN, 'client opened')
 
+                if self.hasServerOption(ray.Option.GUI_STATES):
+                    if (self.session.wait_for == ray.WaitFor.NONE
+                            and self.isCapableOf(':optional-gui:')
+                            and not self.start_gui_hidden
+                            and not self.gui_visible
+                            and not self.gui_has_been_visible):
+                        self.sendToSelfAddress('/nsm/client/show_optional_gui')
+
             self.setStatus(ray.ClientStatus.READY)
             #self.message( "Client \"%s\" replied with: %s in %fms"
                             #% (client.name, message,
@@ -451,22 +479,24 @@ class Client(ServerSender, ray.ClientData):
         return "%s (%s)" % (self.name, self.client_id)
 
     def setNetworkProperties(self, net_daemon_url, net_session_root):
-        if not self.isCapableOf(':ray-network:'):
+        if self.protocol != ray.Protocol.RAY_NET:
             return
 
-        if (net_daemon_url == self.net_daemon_url
-                and net_session_root == self.net_session_root):
-            return
+        self.ray_net.daemon_url = net_daemon_url
+        self.ray_net.running_daemon_url = net_daemon_url
+        self.ray_net.session_root = net_session_root
+        self.ray_net.running_session_root = net_session_root
+        self.sendGuiClientProperties()
 
-        self.net_daemon_url = net_daemon_url
-        self.net_session_root = net_session_root
-
-        self.arguments = '--daemon-url %s --net-session-root "%s"' % (
-                            self.net_daemon_url,
-                            self.net_session_root.replace('"', '\\"'))
+    def getRayNetArgumentsLine(self)->str:
+        if self.protocol != ray.Protocol.RAY_NET:
+            return ''
+        return '--daemon-url %s --net-session-root "%s"' % (
+                self.ray_net.daemon_url,
+                self.ray_net.session_root.replace('"', '\\"'))
 
     def netDaemonOutOfTime(self):
-        self.net_duplicate_state = -1
+        self.ray_net.duplicate_state = -1
 
         if self.session.wait_for == ray.WaitFor.DUPLICATE_FINISH:
             self.session.endTimerIfLastExpected(self)
@@ -493,10 +523,10 @@ class Client(ServerSender, ray.ClientData):
                     and self.client_id[1:4].isupper())
 
     def getJackClientName(self):
-        if self.executable_path == 'ray-network':
-            # ray-network will use jack_client_name for template
+        if self.protocol == ray.Protocol.RAY_NET:
+            # ray-net will use jack_client_name for template
             # quite dirty, but this is the easier way
-            return self.net_session_template
+            return self.ray_net.session_template
 
         # return same jack_client_name as NSM does
         # if client seems to have been made by NSM itself
@@ -534,7 +564,7 @@ class Client(ServerSender, ray.ClientData):
         return ''
 
     def getProjectPath(self):
-        if self.executable_path == 'ray-network':
+        if self.protocol == ray.Protocol.RAY_NET:
             return self.session.getShortPath()
 
         if self.prefix_mode == ray.PrefixMode.SESSION_NAME:
@@ -615,6 +645,9 @@ class Client(ServerSender, ray.ClientData):
         self.session.setRenameable(False)
 
         self.last_dirty = 0.00
+        self.gui_has_been_visible = False
+        self.gui_visible = False
+        self.show_gui_ordered = False
 
         if self.is_dummy:
             self.sendErrorToCaller(OSC_SRC_START, ray.Err.GENERAL_ERROR,
@@ -622,6 +655,13 @@ class Client(ServerSender, ray.ClientData):
                     % self.guiMsgStyle())
             return
 
+        if (self.protocol == ray.Protocol.RAY_NET
+                and not self.session.path.startswith(self.session.root + '/')):
+            self.sendErrorToCaller(OSC_SRC_START, ray.Err.GENERAL_ERROR,
+                _translate('GUIMSG',
+                    "Impossible to run Ray-Net client when session is not in root folder"))
+            return
+
         if self.scripter.start(ray.Command.START, src_addr,
                                self._osc_srcs[OSC_SRC_START]):
             self.setStatus(ray.ClientStatus.SCRIPT)
@@ -631,6 +671,21 @@ class Client(ServerSender, ray.ClientData):
 
         arguments = []
 
+        if self.protocol == ray.Protocol.RAY_NET:
+            server = self.getServer()
+            if not server:
+                return
+
+            arguments += ['--net-daemon-id', str(server.net_daemon_id)]
+            if self.ray_net.daemon_url:
+                arguments += ['--daemon-url', self.ray_net.daemon_url]
+                if self.ray_net.session_root:
+                    arguments += ['--session-root', self.ray_net.session_root]
+            self.ray_net.running_daemon_url = self.ray_net.daemon_url
+            self.ray_net.running_session_root = self.ray_net.session_root
+            self.process.start(ray.RAYNET_BIN, arguments)
+            return
+
         if self.tmp_arguments:
             arguments += shlex.split(self.tmp_arguments)
 
@@ -684,10 +739,6 @@ class Client(ServerSender, ray.ClientData):
         if self.arguments:
             arguments += shlex.split(arguments_line)
 
-        if self.hasServer() and self.executable_path == 'ray-network':
-            arguments.append('--net-daemon-id')
-            arguments.append(str(self.getServer().net_daemon_id))
-
         self.running_executable = self.executable_path
         self.running_arguments = self.arguments
 
@@ -948,7 +999,7 @@ class Client(ServerSender, ray.ClientData):
 
     def tellClientSessionIsLoaded(self):
         if self.active and not self.isDumbClient():
-            Terminal.message("Telling client %s that session is loaded."
+            self.message("Telling client %s that session is loaded."
                              % self.name)
             self.sendToSelfAddress("/nsm/client/session_is_loaded")
 
@@ -993,7 +1044,7 @@ class Client(ServerSender, ray.ClientData):
                 QTimer.singleShot(300, self.rayHackSaved)
 
             elif self.canSaveNow():
-                Terminal.message("Telling %s to save" % self.name)
+                self.message("Telling %s to save" % self.name)
                 self.sendToSelfAddress("/nsm/client/save")
 
                 self.pending_command = ray.Command.SAVE
@@ -1059,7 +1110,7 @@ class Client(ServerSender, ray.ClientData):
             self.sendReplyToCaller(OSC_SRC_STOP, 'client stopped.')
 
     def quit(self):
-        Terminal.message("Commanding %s to quit" % self.name)
+        self.message("Commanding %s to quit" % self.name)
         if self.isRunning():
             self.pending_command = ray.Command.STOP
             self.terminate()
@@ -1086,11 +1137,18 @@ class Client(ServerSender, ray.ClientData):
         self.description = new_client.description
         self._from_nsm_file = new_client._from_nsm_file
 
+        self._desktop_label = new_client._desktop_label
+        self._desktop_description = new_client._desktop_description
+        self._desktop_icon = new_client._desktop_icon
+
+        #self.gui_visible = new_client.gui_visible
+        self.gui_has_been_visible = self.gui_visible
+
     def switch(self):
         jack_client_name = self.getJackClientName()
         client_project_path = self.getProjectPath()
 
-        Terminal.message("Commanding %s to switch \"%s\""
+        self.message("Commanding %s to switch \"%s\""
                          % (self.name, client_project_path))
 
         self.sendToSelfAddress("/nsm/client/open", client_project_path,
@@ -1099,19 +1157,49 @@ class Client(ServerSender, ray.ClientData):
         self.pending_command = ray.Command.OPEN
         self.sendGuiClientProperties()
         self.setStatus(ray.ClientStatus.SWITCH)
+        if self.isCapableOf(':optional-gui:'):
+            self.sendGui('/ray/gui/client/gui_visible',
+                         self.client_id, int(self.gui_visible))
+
+    def canSwitchWith(self, other_client)->bool:
+        if self.protocol == ray.Protocol.RAY_HACK:
+            return False
+
+        if self.protocol != other_client.protocol:
+            return False
+
+        if not ((self.active and self.isCapableOf(':switch:'))
+                or (self.isDumbClient() and self.isRunning())):
+            return False
+
+        if self.protocol == ray.Protocol.RAY_NET:
+            return bool(self.ray_net.running_daemon_url
+                            == other_client.ray_net.daemon_url
+                        and self.ray_net.running_session_root
+                            == other_client.ray_net.session_root)
+
+        return bool(self.running_executable == other_client.executable_path
+                    and self.running_arguments == other_client.arguments)
 
     def sendGuiClientProperties(self, removed=False):
         ad = '/ray/gui/client/update' if self.sent_to_gui else '/ray/gui/client/new'
+        hack_ad = '/ray/gui/client/ray_hack_update'
+        net_ad = '/ray/gui/client/ray_net_update'
 
         if removed:
             ad = '/ray/gui/trash/add'
+            hack_ad = '/ray/gui/trash/ray_hack_update'
+            net_ad = '/ray/gui/trash/ray_net_update'
 
         self.sendGui(ad, *ray.ClientData.spreadClient(self))
-        if not removed:
-            if self.protocol == ray.Protocol.RAY_HACK:
-                self.sendGui('/ray/gui/client/ray_hack_update',
-                             self.client_id,
-                             *self.ray_hack.spread())
+        if self.protocol == ray.Protocol.RAY_HACK:
+            self.sendGui(hack_ad,
+                         self.client_id,
+                         *self.ray_hack.spread())
+        elif self.protocol == ray.Protocol.RAY_NET:
+            self.sendGui(net_ad,
+                         self.client_id,
+                         *self.ray_net.spread())
 
         self.sent_to_gui = True
 
@@ -1178,16 +1266,17 @@ class Client(ServerSender, ray.ClientData):
                     if value.isdigit() and 0 <= int(value) <= 2:
                         self.ray_hack.no_save_level = int(value)
 
+            elif self.protocol == ray.Protocol.RAY_NET:
+                if prop == 'net_daemon_url':
+                    self.ray_net.daemon_url = value
+                elif prop == 'net_session_root':
+                    self.ray_net.session_root = value
+                elif prop == 'net_session_template':
+                    self.ray_net.session_template = value
 
         self.sendGuiClientProperties()
 
     def getPropertiesMessage(self):
-        protocol_str = 'NSM'
-        if self.protocol == ray.Protocol.RAY_HACK:
-            protocol_str = 'Ray-Hack'
-        elif self.protocol == ray.Protocol.RAY_NET:
-            protocol_str = 'Ray-Net'
-
         message = """client_id:%s
 protocol:%s
 executable:%s
@@ -1200,7 +1289,7 @@ label:%s
 icon:%s
 check_last_save:%i
 ignored_extensions:%s""" % (self.client_id,
-                            protocol_str,
+                            ray.protocolToStr(self.protocol),
                             self.executable_path,
                             self.arguments,
                             self.name,
@@ -1224,7 +1313,12 @@ no_save_level:%i""" % (self.ray_hack.config_file,
                        self.ray_hack.stop_sig,
                        int(self.ray_hack.wait_win),
                        self.ray_hack.no_save_level)
-
+        elif self.protocol == ray.Protocol.RAY_NET:
+            message += """\nnet_daemon_url:%s
+net_session_root:%s
+net_session_template:%s""" % (self.ray_net.daemon_url,
+                              self.ray_net.session_root,
+                              self.ray_net.session_template)
         return message
 
     def prettyClientId(self):
@@ -1453,11 +1547,14 @@ no_save_level:%i""" % (self.ray_hack.config_file,
 
         os.makedirs(template_dir)
 
-        if self.net_daemon_url:
-            self.net_session_template = template_name
-            self.send(Address(self.net_daemon_url),
-                      '/ray/server/save_session_template', self.session.name,
-                      template_name, self.net_session_root)
+        if self.protocol == ray.Protocol.RAY_NET:
+            if self.ray_net.daemon_url:
+                self.ray_net.session_template = template_name
+                self.send(Address(self.ray_net.daemon_url),
+                          '/ray/server/save_session_template',
+                          self.session.name,
+                          template_name,
+                          self.net_session_root)
 
         if client_files:
             self.setStatus(ray.ClientStatus.COPY)
@@ -1541,6 +1638,9 @@ no_save_level:%i""" % (self.ray_hack.config_file,
         file.write(xml.toString())
         file.close()
 
+        self.template_origin = template_name
+        self.sendGuiClientProperties()
+
         self.sendGuiMessage(
             _translate('message', 'Client template %s created')
                 % template_name)
@@ -1712,6 +1812,32 @@ no_save_level:%i""" % (self.ray_hack.config_file,
                         do_rename = False
                         break
 
+                    # only for hydrogen
+                    hydrogen_file = "%s/%s.%s.h2song" % (
+                        project_path, old_prefix, old_client_id)
+                    hydrogen_autosave = "%s/%s.%s.autosave.h2song" % (
+                        project_path, old_prefix, old_client_id)
+
+                    if (os.path.isfile(hydrogen_file)
+                            and os.access(hydrogen_file, os.W_OK)):
+                        new_hydro_file = "%s/%s.%s.h2song" % (
+                            project_path, new_prefix, new_client_id)
+                        if os.path.exists(new_hydro_file):
+                            do_rename = False
+                            break
+
+                        files_to_rename.append((hydrogen_file, new_hydro_file))
+
+                    if (os.path.isfile(hydrogen_autosave)
+                            and os.access(hydrogen_autosave, os.W_OK)):
+                        new_hydro_autosave = "%s/%s.%s.autosave.h2song" % (
+                            project_path, new_prefix, new_client_id)
+                        if os.path.exists(new_hydro_autosave):
+                            do_rename = False
+                            break
+
+                        files_to_rename.append((hydrogen_autosave, new_hydro_autosave))
+
                     # only for ardour
                     ardour_file = "%s/%s.ardour" % (project_path, old_prefix)
                     ardour_bak = "%s/%s.ardour.bak" % (project_path, old_prefix)
@@ -1874,7 +2000,7 @@ no_save_level:%i""" % (self.ray_hack.config_file,
             return
 
         if major > NSM_API_VERSION_MAJOR:
-            Terminal.message(
+            self.message(
                 "Client is using incompatible and more recent "
                 + "API version %i.%i" % (major, minor))
             self.send(src_addr, "/error", path, ray.Err.INCOMPATIBLE_API,
@@ -1895,8 +2021,8 @@ no_save_level:%i""" % (self.ray_hack.config_file,
         if self.executable_path in RS.non_active_clients:
             RS.non_active_clients.remove(self.executable_path)
 
-        Terminal.message("Process has pid: %i" % pid)
-        Terminal.message(
+        self.message("Process has pid: %i" % pid)
+        self.message(
             "The client \"%s\" at \"%s\" " % (self.name, self.addr.url)
             + "informs us it's ready to receive commands.")
 
@@ -1927,19 +2053,13 @@ no_save_level:%i""" % (self.ray_hack.config_file,
         client_project_path = self.getProjectPath()
         jack_client_name = self.getJackClientName()
 
-        if self.isCapableOf(':ray-network:'):
+        if self.protocol == ray.Protocol.RAY_NET:
             client_project_path = self.session.getShortPath()
-            jack_client_name = self.net_session_template
+            jack_client_name = self.ray_net.session_template
 
         self.send(src_addr, "/nsm/client/open", client_project_path,
                   self.session.name, jack_client_name)
 
         self.pending_command = ray.Command.OPEN
 
-        if self.isCapableOf(":optional-gui:"):
-            self.sendGui("/ray/gui/client/has_optional_gui", self.client_id)
-
-            if self.start_gui_hidden:
-                self.send(src_addr, "/nsm/client/hide_optional_gui")
-
         self._last_announce_time = time.time()
