@@ -49,8 +49,11 @@ class MultiDaemonFile:
 
     def openFile(self):
         if not os.path.exists(self.file_path):
-            if not os.path.exists(os.path.dirname(self.file_path)):
-                os.makedirs(os.path.dirname(self.file_path))
+            dir_path = os.path.dirname(self.file_path)
+            if not os.path.exists(dir_path):
+                os.makedirs(dir_path)
+                # give read/write access for all users
+                os.chmod(dir_path, 0o777)
 
             return False
 
@@ -83,6 +86,7 @@ class MultiDaemonFile:
             int(bool(self.server.is_nsm_locked or self.server.not_default)))
         element.setAttribute('has_gui', int(self.server.hasGui()))
         element.setAttribute('version', ray.VERSION)
+        element.setAttribute('local_gui_pids', self.server.getLocalGuiPidList())
 
     def update(self):
         has_dirty_pid = False
