@@ -17,8 +17,17 @@ instance = None
 signaler = Signaler.instance()
 
 
-def pathIsValid(path):
-    return not bool('../' in path)
+def pathIsValid(path: str)->bool:
+    if path.startswith(('./', '../')):
+        return False
+
+    for forbidden in ('//', '/./', '/../'):
+        if forbidden in path:
+            return False
+
+    if path.endswith(('/.', '/..')):
+        return False
+    return True
 
 def ifDebug(string):
     if CommandLineArgs.debug:
@@ -46,6 +55,9 @@ class Controller:
     pid = 0
 
 
+class GuiAdress(liblo.Address):
+    gui_pid = 0
+
 # Osc server thread separated in many classes for confort.
 
 # ClientCommunicating contains NSM protocol.
@@ -65,6 +77,7 @@ class ClientCommunicating(liblo.ServerThread):
         self.net_master_daemon_url = ''
         self.net_daemon_id = random.randint(1, 999999999)
         self.list_asker_addr = None
+        self.options = 0
 
     @ray_method('/osc/ping', '')
     def oscPing(self, path, args, types, src_addr):
@@ -72,7 +85,10 @@ class ClientCommunicating(liblo.ServerThread):
 
     @ray_method('/reply', None)
     def reply(self, path, args, types, src_addr):
-        if not (len(args) >= 2 and ray.areTheyAllString(args)):
+        if not ray.areTheyAllString(args):
+            self.unknownMessage(path, types, src_addr)
+
+        if not len(args) >= 1:
             self.unknownMessage(path, types, src_addr)
             return False
 
@@ -95,6 +111,8 @@ class ClientCommunicating(liblo.ServerThread):
             return False
 
         if not len(args) == 2:
+            # assume this is a normal client, not a net_daemon
+            self.unknownMessage(path, types, src_addr)
             return False
 
     @ray_method('/error', 'sis')
@@ -110,6 +128,11 @@ class ClientCommunicating(liblo.ServerThread):
                           'User action dialog aborted !')
             return False
 
+    @ray_method('/minor_error', 'sis')
+    def minor_error(self, path, args, types, src_addr):
+        # prevent minor_error to minor_error loop in daemon <-> daemon communication
+        pass
+
     # SERVER_CONTROL messages
     # following messages only for :server-control: capability
     @ray_method('/nsm/server/add', 's')
@@ -280,6 +303,7 @@ class ClientCommunicating(liblo.ServerThread):
         Terminal.message("Client '%s' sends gui shown" % client.client_id)
 
         client.gui_visible = True
+        client.gui_has_been_visible = True
 
         self.sendGui("/ray/gui/client/gui_visible", client.client_id,
                      int(client.gui_visible))
@@ -301,31 +325,34 @@ class OscServerThread(ClientCommunicating):
     def __init__(self, session, osc_num=0):
         ClientCommunicating.__init__(self, session, osc_num)
 
-        self.option_save_from_client = RS.settings.value(
-            'daemon/save_all_from_saved_client', True, type=bool)
-        self.option_bookmark_session = RS.settings.value(
-            'daemon/bookmark_session_folder', True, type=bool)
-        self.option_desktops_memory = RS.settings.value(
-            'daemon/desktops_memory', False, type=bool)
-        self.option_snapshots = RS.settings.value(
-            'daemon/auto_snapshot', True, type=bool)
-        self.option_session_scripts = RS.settings.value(
-            'daemon/session_scripts', True, type=bool)
+        self.options_dict = {
+            'save_from_client': ray.Option.SAVE_FROM_CLIENT,
+            'bookmark_session_folder': ray.Option.BOOKMARK_SESSION,
+            'desktops_memory': ray.Option.DESKTOPS_MEMORY,
+            'snapshots': ray.Option.SNAPSHOTS,
+            'session_scripts': ray.Option.SESSION_SCRIPTS,
+            'gui_states': ray.Option.GUI_STATES}
 
-        self.option_has_wmctrl = bool(shutil.which('wmctrl'))
-        if not self.option_has_wmctrl:
-            self.option_desktops_memory = False
-
-        self.option_has_git = bool(shutil.which('git'))
-        if not self.option_has_git:
-            self.option_snapshots = False
+        self.options = RS.settings.value(
+            'daemon/options',
+            ray.Option.BOOKMARK_SESSION
+                + ray.Option.SNAPSHOTS
+                + ray.Option.SESSION_SCRIPTS
+                + ray.Option.GUI_STATES,
+            type=int)
 
         if CommandLineArgs.no_options:
-            self.option_save_from_client = False
-            self.option_bookmark_session = False
-            self.option_desktops_memory = False
-            self.option_snapshots = False
-            self.option_session_scripts = False
+            self.options = 0
+
+        if shutil.which('wmctrl'):
+            self.options |= ray.Option.HAS_WMCTRL
+        elif self.options & ray.Option.HAS_WMCTRL:
+            self.options -= ray.Option.HAS_WMCTRL
+
+        if shutil.which('git'):
+            self.options |= ray.Option.HAS_GIT
+        elif self.options & ray.Option.HAS_GIT:
+            self.options -= ray.Option.HAS_GIT
 
         global instance
         instance = self
@@ -336,12 +363,14 @@ class OscServerThread(ClientCommunicating):
 
     @ray_method('/ray/server/gui_announce', 'sisii')
     def rayGuiGui_announce(self, path, args, types, src_addr):
-        version = args[0]
-        nsm_locked = bool(args[1])
+        (version, int_nsm_locked, net_master_daemon_url,
+         gui_pid, net_daemon_id) = args
+
+        nsm_locked = bool(int_nsm_locked)
         is_net_free = True
 
         if nsm_locked:
-            self.net_master_daemon_url = args[2]
+            self.net_master_daemon_url = net_master_daemon_url
             self.is_nsm_locked = True
             self.nsm_locker_url = src_addr.url
 
@@ -349,7 +378,7 @@ class OscServerThread(ClientCommunicating):
                 if not ray.areSameOscPort(gui_addr.url, src_addr.url):
                     self.send(gui_addr, '/ray/gui/server/nsm_locked', 1)
 
-            self.net_daemon_id = args[4]
+            self.net_daemon_id = net_daemon_id
 
             multi_daemon_file = MultiDaemonFile.getInstance()
 
@@ -357,7 +386,7 @@ class OscServerThread(ClientCommunicating):
                 is_net_free = multi_daemon_file.isFreeForRoot(
                     self.net_daemon_id, self.session.root)
 
-        self.announceGui(src_addr.url, nsm_locked, is_net_free)
+        self.announceGui(src_addr.url, nsm_locked, is_net_free, gui_pid)
 
     @ray_method('/ray/server/gui_disannounce', '')
     def rayGuiGui_disannounce(self, path, args, types, src_addr):
@@ -422,6 +451,11 @@ class OscServerThread(ClientCommunicating):
 
     @ray_method('/ray/server/change_root', 's')
     def rayServerChangeRoot(self, path, args, types, src_addr):
+        new_root = args[0]
+        if not(new_root.startswith('/') and pathIsValid(new_root)):
+            self.send(src_addr, '/error', path, ray.Err.CREATE_FAILED,
+                      "invalid session root !")
+
         if self.isOperationPending(src_addr, path):
             self.send(src_addr, '/error', path, ray.Err.OPERATION_PENDING,
                       "Can't change session_root. Operation pending")
@@ -587,27 +621,6 @@ class OscServerThread(ClientCommunicating):
     def rayServerOpenSessionWithoutSaveOff(self, path, args, types, src_addr):
         pass
 
-    @ray_method('/ray/session/save', '')
-    def raySessionSave(self, path, args, types, src_addr):
-        if not self.session.path:
-            self.send(src_addr, "/error", path, ray.Err.NO_SESSION_OPEN,
-                      "No session to save.")
-            return False
-
-    @ray_method('/ray/session/run_step', None)
-    def raySessionProcessStep(self, path, args, types, src_addr):
-        if not ray.areTheyAllString(args):
-            self.unknownMessage(path, types, src_addr)
-            return False
-
-    @ray_method('/ray/session/save_as_template', 's')
-    def raySessionSaveAsTemplate(self, path, args, types, src_addr):
-        template_name = args[0]
-        if '/' in template_name:
-            self.send(src_addr, "/error", path, ray.Err.CREATE_FAILED,
-                      "Invalid session template name.")
-            return False
-
     @ray_method('/ray/server/save_session_template', 'ss')
     def rayServerSaveSessionTemplate(self, path, args, types, src_addr):
         #save as template an not loaded session
@@ -615,12 +628,12 @@ class OscServerThread(ClientCommunicating):
 
         if not pathIsValid(session_name):
             self.send(src_addr, "/error", path, ray.Err.CREATE_FAILED,
-                    "Invalid template name.")
+                    "Invalid session name.")
             return False
 
         if '/' in template_name:
             self.send(src_addr, "/error", path, ray.Err.CREATE_FAILED,
-                      "Invalid session name.")
+                      "Invalid template name.")
             return False
 
     @ray_method('/ray/server/rename_session', 'ss')
@@ -660,70 +673,68 @@ class OscServerThread(ClientCommunicating):
             return
         self.sendGui('/ray/gui/script_user_action', args[0])
 
+    # set option from GUI
+    @ray_method('/ray/server/set_option', 'i')
+    def rayServerSetOption(self, path, args, types, src_addr):
+        option = args[0]
+        self.setOption(option)
+
+        for gui_addr in self.gui_list:
+            if not ray.areSameOscPort(gui_addr.url, src_addr.url):
+                self.send(gui_addr, '/ray/gui/server/options', self.options)
+
+    # set options from ray_control
     @ray_method('/ray/server/set_options', None)
     def rayServerSetOptions(self, path, args, types, src_addr):
         if not ray.areTheyAllString(args):
             self.unknownMessage(path, types, src_addr)
             return False
 
-        for option in args:
+        for option_str in args:
             option_value = True
-            if option.startswith('not_'):
+            if option_str.startswith('not_'):
                 option_value = False
-                option = option.replace('not_', '', 1)
+                option_str = option_str.replace('not_', '', 1)
 
-            if option == 'save_from_client':
-                self.option_save_from_client = option_value
-            elif option == 'bookmark_session_folder':
-                self.option_bookmark_session = option_value
-            elif option == 'desktops_memory':
-                if option_value and not self.option_has_wmctrl:
-                    self.send(src_addr, '/minor_error', path,
-                        "wmctrl is not present. Impossible to activate 'desktops_memory' option")
-                    continue
-                self.option_desktops_memory = option_value
-            elif option == 'snapshots':
-                if option_value and not self.option_has_git:
-                    self.send(src_addr, '/minor_error', path,
-                        "git is not present. Impossible to activate 'snapshots' option")
-                    continue
-                self.option_snapshots = option_value
-            elif option == 'session_scripts':
-                self.option_session_scripts = option_value
+            if option_str in self.options_dict:
+                option = self.options_dict[option_str]
+                if option_value:
+                    if (option == ray.Option.DESKTOPS_MEMORY
+                            and not self.options & ray.Option.HAS_WMCTRL):
+                        self.send(src_addr, '/minor_error', path,
+                            "wmctrl is not present. Impossible to activate 'desktops_memory' option")
+                        continue
+                    if (option == ray.Option.SNAPSHOTS
+                            and not self.options & ray.Option.HAS_GIT):
+                        self.send(src_addr, '/minor_error', path,
+                            "git is not present. Impossible to activate 'snapshots' option")
+                        continue
 
-        options = self.getOptions()
+                if not option_value:
+                    option = -option
+                self.setOption(option)
 
         for gui_addr in self.gui_list:
             if not ray.areSameOscPort(gui_addr.url, src_addr.url):
-                self.send(gui_addr, '/ray/gui/server/options', options)
+                self.send(gui_addr, '/ray/gui/server/options', self.options)
 
         self.send(src_addr, '/reply', path, 'Options set')
 
     @ray_method('/ray/server/has_option', 's')
     def rayServerHasOption(self, path, args, types, src_addr):
-        option = args[0]
+        option_str = args[0]
         option_value = False
 
-        if option == 'save_from_client':
-            option_value = self.option_save_from_client
-        elif option == 'bookmark_session_folder':
-            option_value = self.option_bookmark_session
-        elif option == 'desktops_memory':
-            option_value = self.option_desktops_memory
-        elif option == 'snapshots':
-            option_value = self.option_snapshots
-        elif option == 'session_scripts':
-            option_value = self.option_session_scripts
-        else:
+        if option_str not in self.options_dict:
             self.send(src_addr, '/error', path, ray.Err.GENERAL_ERROR,
-                      "option \"%s\" doesn't exists" % option)
+                      "option \"%s\" doesn't exists" % option_str)
             return
 
-        if option_value:
+        if self.options & self.options_dict[option_str]:
             self.send(src_addr, '/reply', path, 'Has option')
         else:
             self.send(src_addr, '/error', path, ray.Err.GENERAL_ERROR,
-                      "Option %s is not currently used" % option)
+                      "Option %s is not currently used" % option_str)
 
     @ray_method('/ray/server/exotic_action', 's')
     def rayServerExoticAction(self, path, args, types, src_addr):
@@ -744,6 +755,27 @@ class OscServerThread(ClientCommunicating):
         elif action == 'unset_jack_checker_autostart':
             os.remove("%s/%s" % (autostart_dir, desk_file))
 
+    @ray_method('/ray/session/save', '')
+    def raySessionSave(self, path, args, types, src_addr):
+        if not self.session.path:
+            self.send(src_addr, "/error", path, ray.Err.NO_SESSION_OPEN,
+                      "No session to save.")
+            return False
+
+    @ray_method('/ray/session/run_step', None)
+    def raySessionProcessStep(self, path, args, types, src_addr):
+        if not ray.areTheyAllString(args):
+            self.unknownMessage(path, types, src_addr)
+            return False
+
+    @ray_method('/ray/session/save_as_template', 's')
+    def raySessionSaveAsTemplate(self, path, args, types, src_addr):
+        template_name = args[0]
+        if '/' in template_name or template_name == '.':
+            self.send(src_addr, "/error", path, ray.Err.CREATE_FAILED,
+                      "Invalid session template name.")
+            return False
+
     @ray_method('/ray/session/get_session_name', '')
     def raySessionGetSessionName(self, path, args, types, src_addr):
         if not self.session.path:
@@ -833,9 +865,12 @@ class OscServerThread(ClientCommunicating):
 
     @ray_method('/ray/session/set_notes', 's')
     def raySessionSetNotes(self, path, args, types, src_addr):
+        self.session.notes = args[0]
+
         for gui_addr in self.gui_list:
             if not ray.areSameOscPort(gui_addr.url, src_addr.url):
-                self.send(gui_addr, '/ray/gui/session/notes', args[0])
+                self.send(gui_addr, '/ray/gui/session/notes',
+                          self.session.notes)
 
     @ray_method('/ray/session/get_notes', '')
     def raySessionGetNotes(self, path, args, types, src_addr):
@@ -880,13 +915,17 @@ class OscServerThread(ClientCommunicating):
     def rayServerAddClientTemplate(self, path, args, types, src_addr):
         pass
 
-    @ray_method('/ray/session/add_factory_client_template', 's')
+    @ray_method('/ray/session/add_factory_client_template', None)
     def raySessionAddFactoryClientTemplate(self, path, args, types, src_addr):
-        pass
+        if not (args and ray.areTheyAllString(args)):
+            self.unknownMessage(path, types, src_addr)
+            return False
 
-    @ray_method('/ray/session/add_user_client_template', 's')
+    @ray_method('/ray/session/add_user_client_template', None)
     def raySessionAddUserClientTemplate(self, path, args, types, src_addr):
-        pass
+        if not (args and ray.areTheyAllString(args)):
+            self.unknownMessage(path, types, src_addr)
+            return False
 
     @ray_method('/ray/session/reorder_clients', None)
     def rayServerReorderClients(self, path, args, types, src_addr):
@@ -913,6 +952,28 @@ class OscServerThread(ClientCommunicating):
             self.unknownMessage(path, types, src_addr)
             return False
 
+    @ray_method('/ray/session/show_notes', '')
+    def raySessionShowNotes(self, path, args, types, src_addr):
+        if not self.session.path:
+            self.send(src_addr, '/error', path, ray.Err.NO_SESSION_OPEN,
+                      "No session to show notes")
+            return False
+
+        self.session.notes_shown = True
+        self.sendGui('/ray/gui/session/notes_shown')
+        self.send(src_addr, '/reply', path, 'notes shown')
+
+    @ray_method('/ray/session/hide_notes', '')
+    def raySessionHideNotes(self, path, args, types, src_addr):
+        if not self.session.path:
+            self.send(src_addr, '/error', path, ray.Err.NO_SESSION_OPEN,
+                      "No session to hide notes")
+            return False
+
+        self.session.notes_shown = False
+        self.sendGui('/ray/gui/session/notes_hidden')
+        self.send(src_addr, '/reply', path, 'notes hidden')
+
     @ray_method('/ray/session/list_clients', None)
     def raySessionListClients(self, path, args, types, src_addr):
         if not ray.areTheyAllString(args):
@@ -967,10 +1028,14 @@ class OscServerThread(ClientCommunicating):
     def rayGuiClientUpdateProperties(self, path, args, types, src_addr):
         pass
 
-    @ray_method('/ray/client/update_ray_hack_properties', 'ssiiiisi')
+    @ray_method('/ray/client/update_ray_hack_properties', 's' + ray.RayHack.sisi())
     def rayClientUpdateRayHackProperties(self, path, args, types, src_addr):
         pass
 
+    @ray_method('/ray/client/update_ray_net_properties', 's' + ray.RayNet.sisi())
+    def rayClientUpdateRayNetProperties(self, path, args, types, src_addr):
+        pass
+
     @ray_method('/ray/client/get_properties', 's')
     def rayClientGetProperties(self, path, args, types, src_addr):
         pass
@@ -1067,56 +1132,6 @@ class OscServerThread(ClientCommunicating):
     def rayDuplicateState(self, path, args, types, src_addr):
         pass
 
-    @ray_method('/ray/option/save_from_client', 'i')
-    def rayOptionSaveFromClient(self, path, args, types, src_addr):
-        self.option_save_from_client = bool(args[0])
-
-        options = self.getOptions()
-
-        for gui_addr in self.gui_list:
-            if not ray.areSameOscPort(gui_addr.url, src_addr.url):
-                self.send(gui_addr, '/ray/gui/server/options', options)
-
-    @ray_method('/ray/option/bookmark_session_folder', 'i')
-    def rayOptionBookmarkSessionFolder(self, path, args, types, src_addr):
-        self.option_bookmark_session = bool(args[0])
-
-        options = self.getOptions()
-
-        for gui_addr in self.gui_list:
-            if not ray.areSameOscPort(gui_addr.url, src_addr.url):
-                self.send(gui_addr, '/ray/gui/server/options', options)
-
-    @ray_method('/ray/option/desktops_memory', 'i')
-    def rayOptionDesktopsMemory(self, path, args, types, src_addr):
-        self.option_desktops_memory = bool(args[0])
-
-        options = self.getOptions()
-
-        for gui_addr in self.gui_list:
-            if not ray.areSameOscPort(gui_addr.url, src_addr.url):
-                self.send(gui_addr, '/ray/gui/server/options', options)
-
-    @ray_method('/ray/option/snapshots', 'i')
-    def rayOptionSnapshots(self, path, args, types, src_addr):
-        self.option_snapshots = bool(args[0])
-
-        options = self.getOptions()
-
-        for gui_addr in self.gui_list:
-            if not ray.areSameOscPort(gui_addr.url, src_addr.url):
-                self.send(gui_addr, '/ray/gui/server/options', options)
-
-    @ray_method('/ray/option/session_scripts', 'i')
-    def rayOptionSessionScripts(self, path, args, types, src_addr):
-        self.option_session_scripts = bool(args[0])
-
-        options = self.getOptions()
-
-        for gui_addr in self.gui_list:
-            if not ray.areSameOscPort(gui_addr.url, src_addr.url):
-                self.send(gui_addr, '/ray/gui/server/options', options)
-
     @ray_method('/ray/favorites/add', 'ssi')
     def rayFavoriteAdd(self, path, args, types, src_addr):
         name, icon, int_factory = args
@@ -1129,9 +1144,7 @@ class OscServerThread(ClientCommunicating):
         else:
             RS.favorites.append(ray.Favorite(name, icon, bool(int_factory)))
 
-        for gui_addr in self.gui_list:
-            if not ray.areSameOscPort(gui_addr.url, src_addr.url):
-                self.send(gui_addr, '/ray/gui/favorites/added', *args)
+        self.sendGui('/ray/gui/favorites/added', *args)
 
     @ray_method('/ray/favorites/remove', 'si')
     def rayFavoriteRemove(self, path, args, types, src_addr):
@@ -1143,9 +1156,7 @@ class OscServerThread(ClientCommunicating):
                 RS.favorites.remove(favorite)
                 break
 
-        for gui_addr in self.gui_list:
-            if not ray.areSameOscPort(gui_addr.url, src_addr.url):
-                self.send(gui_addr, '/ray/gui/favorites/removed', *args)
+        self.sendGui('/ray/gui/favorites/removed', *args)
 
     @ray_method(None, None)
     def noneMethod(self, path, args, types, src_addr):
@@ -1214,26 +1225,12 @@ class OscServerThread(ClientCommunicating):
 
         self.sendGui('/ray/gui/session/renameable', 1)
 
-    def getOptions(self):
-        options = (
-            ray.Option.NSM_LOCKED * self.is_nsm_locked
-            + ray.Option.SAVE_FROM_CLIENT * self.option_save_from_client
-            + ray.Option.BOOKMARK_SESSION * self.option_bookmark_session
-            + ray.Option.HAS_WMCTRL * self.option_has_wmctrl
-            + ray.Option.DESKTOPS_MEMORY * self.option_desktops_memory
-            + ray.Option.HAS_GIT * self.option_has_git
-            + ray.Option.SNAPSHOTS * self.option_snapshots
-            + ray.Option.SESSION_SCRIPTS * self.option_session_scripts)
-
-        return options
-
-    def announceGui(self, url, nsm_locked=False, is_net_free=True):
-        gui_addr = liblo.Address(url)
-
-        options = self.getOptions()
+    def announceGui(self, url, nsm_locked=False, is_net_free=True, gui_pid=0):
+        gui_addr = GuiAdress(url)
+        gui_addr.gui_pid = gui_pid
 
         self.send(gui_addr, "/ray/gui/server/announce", ray.VERSION,
-                  self.server_status, options, self.session.root,
+                  self.server_status, self.options, self.session.root,
                   int(is_net_free))
 
         self.send(gui_addr, "/ray/gui/server/status", self.server_status)
@@ -1255,16 +1252,37 @@ class OscServerThread(ClientCommunicating):
                           '/ray/gui/client/ray_hack_update',
                           client.client_id,
                           *client.ray_hack.spread())
+            elif client.protocol == ray.Protocol.RAY_NET:
+                self.send(gui_addr,
+                          '/ray/gui/client/ray_net_update',
+                          client.client_id,
+                          *client.ray_net.spread())
 
             self.send(gui_addr, "/ray/gui/client/status",
                       client.client_id, client.status)
 
             if client.isCapableOf(':optional-gui:'):
-                self.send(gui_addr, '/ray/gui/client/has_optional_gui',
-                          client.client_id)
+                #self.send(gui_addr, '/ray/gui/client/has_optional_gui',
+                          #client.client_id)
 
                 self.send(gui_addr, '/ray/gui/client/gui_visible',
-                          client.client_id, client.gui_visible)
+                          client.client_id, int(client.gui_visible))
+
+            if client.isCapableOf(':dirty:'):
+                self.send(gui_addr, '/ray/gui/is_dirty', client.dirty)
+
+        for trashed_client in self.session.trashed_clients:
+            self.send(gui_addr, '/ray/gui/trash/add',
+                      *trashed_client.spread())
+
+            if trashed_client.protocol == ray.Protocol.RAY_HACK:
+                self.send(gui_addr, '/ray/gui/trash/ray_hack_update',
+                          trashed_client.client_id,
+                          *trashed_client.ray_hack.spread())
+            elif trashed_client.protocol == ray.Protocol.RAY_NET:
+                self.send(gui_addr, '/ray/gui/trash/ray_net_update',
+                          trashed_client.client_id,
+                          *trashed_client.ray_net.spread())
 
         self.gui_list.append(gui_addr)
 
@@ -1279,7 +1297,7 @@ class OscServerThread(ClientCommunicating):
         controller.addr = control_address
         self.controller_list.append(controller)
         self.send(control_address, "/ray/control/server/announce",
-                  ray.VERSION, self.server_status, self.getOptions(),
+                  ray.VERSION, self.server_status, self.options,
                   self.session.root, 1)
 
     def sendControllerMessage(self, message):
@@ -1303,7 +1321,7 @@ class OscServerThread(ClientCommunicating):
             if ray.areOnSameMachine(self.url, gui_addr.url):
                 # we've got a local GUI
                 return 3
-            
+
             has_gui = True
 
         if has_gui:
@@ -1311,8 +1329,21 @@ class OscServerThread(ClientCommunicating):
 
         return 0
 
+    def getLocalGuiPidList(self)->str:
+        pid_list = []
+        for gui_addr in self.gui_list:
+            if ray.areOnSameMachine(gui_addr.url, self.url):
+                pid_list.append(str(gui_addr.gui_pid))
+        return ':'.join(pid_list)
+
     def isGuiAddress(self, addr):
         for gui_addr in self.gui_list:
             if ray.areSameOscPort(gui_addr.url, addr.url):
                 return True
         return False
+
+    def setOption(self, option: int):
+        if option > 0:
+            self.options |= option
+        else:
+            self.options &= ~option
