@@ -52,6 +52,8 @@ class Session(ServerSender):
         self.future_session_name = ""
         self.notes = ""
         self.future_notes = ""
+        self.notes_shown = False
+        self.future_notes_shown = False
         self.load_locked = False
 
         self.is_renameable = True
@@ -90,7 +92,11 @@ class Session(ServerSender):
         if self.is_dummy and not even_dummy:
             return
 
-        Terminal.message(string)
+        server = self.getServer()
+        if server:
+            Terminal.message(string, server.port)
+        else:
+            Terminal.message(string)
 
     def setRoot(self, session_root):
         if self.path:
@@ -128,7 +134,7 @@ class Session(ServerSender):
 
         if self.path:
             server = self.getServer()
-            if server and server.option_bookmark_session:
+            if server and server.options & ray.Option.BOOKMARK_SESSION:
                 self.bookmarker.setDaemonPort(server.port)
                 self.bookmarker.makeAll(self.path)
 
@@ -138,6 +144,7 @@ class Session(ServerSender):
         self.future_session_name = ''
         self.future_trashed_clients.clear()
         self.future_notes = ""
+        self.future_notes_shown = False
 
     def getShortPath(self):
         if self.path.startswith("%s/" % self.root):
@@ -189,23 +196,28 @@ class Session(ServerSender):
 
         client.setStatus(ray.ClientStatus.REMOVED)
 
-        if client.isRayHack():
-            client_dir = client.getProjectPath()
-            if os.path.isdir(client_dir):
-                if os.listdir(client_dir):
-                    self.trashed_clients.append(client)
-                    client.sendGuiClientProperties(removed=True)
-                else:
-                    try:
-                        os.removedirs(client_dir)
-                    except:
-                        self.trashed_clients.append(client)
-                        client.sendGuiClientProperties(removed=True)
+        ## Theses lines are commented because finally choice is to
+        ## always send client to trash
+        ## comment self.trashed_client.append(client) if choice is reversed !!!
+        #if client.isRayHack():
+            #client_dir = client.getProjectPath()
+            #if os.path.isdir(client_dir):
+                #if os.listdir(client_dir):
+                    #self.trashed_clients.append(client)
+                    #client.sendGuiClientProperties(removed=True)
+                #else:
+                    #try:
+                        #os.removedirs(client_dir)
+                    #except:
+                        #self.trashed_clients.append(client)
+                        #client.sendGuiClientProperties(removed=True)
 
-        elif client.getProjectFiles() or client.net_daemon_url:
-            self.trashed_clients.append(client)
-            client.sendGuiClientProperties(removed=True)
+        #elif client.getProjectFiles() or client.net_daemon_url:
+            #self.trashed_clients.append(client)
+            #client.sendGuiClientProperties(removed=True)
 
+        self.trashed_clients.append(client)
+        client.sendGuiClientProperties(removed=True)
         self.clients.remove(client)
 
     def removeClient(self, client):
@@ -400,7 +412,7 @@ class Session(ServerSender):
                     client_newlist.append(client)
                     break
 
-        if len(client_ids_list) != len(self.clients):
+        if len(client_newlist) != len(self.clients):
             if src_addr:
                 self.send(src_addr, '/error', src_path, ray.Err.GENERAL_ERROR,
                           "%s clients are missing or incorrect" \
@@ -414,6 +426,9 @@ class Session(ServerSender):
         if src_addr:
             self.answer(src_addr, src_path, "clients reordered")
 
+        self.sendGui('/ray/gui/session/sort_clients',
+                     *[c.client_id for c in self.clients])
+
     def isPathInASessionDir(self, spath):
         if self.isNsmLocked() and os.getenv('NSM_URL'):
             return False
@@ -643,8 +658,7 @@ class OperatingSession(Session):
             if len(next_item) > 1:
                 arguments = next_item[1:]
 
-        server = self.getServer()
-        if (server and server.option_session_scripts
+        if (self.hasServerOption(ray.Option.SESSION_SCRIPTS)
                 and not self.step_scripter.isRunning()
                 and self.path and not from_run_step):
             for step_string in ('load', 'save', 'close'):
@@ -725,8 +739,7 @@ class OperatingSession(Session):
             self.window_waiter.stop()
             return
 
-        server = self.getServer()
-        if server and server.option_has_wmctrl:
+        if self.hasServerOption(ray.Option.HAS_WMCTRL):
             self.desktops_memory.setActiveWindowList()
             for client in self.clients:
                 if client.ray_hack_waiting_win:
@@ -917,6 +930,8 @@ class OperatingSession(Session):
         p = xml.createElement('RAYSESSION')
         p.setAttribute('VERSION', ray.VERSION)
         p.setAttribute('name', self.name)
+        if self.notes_shown:
+            p.setAttribute('notes_shown', 'true')
 
         xml_cls = xml.createElement('Clients')
         xml_rmcls = xml.createElement('RemovedClients')
@@ -943,7 +958,8 @@ class OperatingSession(Session):
 
             xml_rmcls.appendChild(cl)
 
-        if self.hasServer() and self.getServer().option_desktops_memory:
+        if (self.hasServer()
+                and self.getServer().options & ray.Option.DESKTOPS_MEMORY):
             self.desktops_memory.save()
 
         for win in self.desktops_memory.saved_windows:
@@ -980,13 +996,13 @@ class OperatingSession(Session):
                 notes_file.write(self.notes)
                 notes_file.close()
             except:
-                Terminal.message("unable to save notes in %s"
+                self.message("unable to save notes in %s"
                                  % full_notes_path)
         elif os.path.isfile(full_notes_path):
             try:
                 os.remove(full_notes_path)
             except:
-                Terminal.message("unable to remove %s" % full_notes_path)
+                self.message("unable to remove %s" % full_notes_path)
 
         self.sendGuiMessage(_translate('GUIMSG', "Session '%s' saved.")
                                 % self.getShortPath())
@@ -1018,8 +1034,7 @@ class OperatingSession(Session):
     def snapshot(self, snapshot_name='', rewind_snapshot='',
                  force=False, outing=False):
         if not force:
-            server = self.getServer()
-            if not (server and server.option_snapshots
+            if not (self.hasServerOption(ray.Option.SNAPSHOTS)
                     and not self.snapshoter.isAutoSnapshotPrevented()
                     and self.snapshoter.hasChanges()):
                 self.nextFunction()
@@ -1075,8 +1090,7 @@ class OperatingSession(Session):
     def closeNoSaveClients(self):
         self.cleanExpected()
 
-        server = self.getServer()
-        if server and server.option_has_wmctrl:
+        if self.hasServerOption(ray.Option.HAS_WMCTRL):
             has_nosave_clients = False
             for client in self.clients:
                 if client.isRunning() and client.noSaveLevel() == 2:
@@ -1125,30 +1139,30 @@ class OperatingSession(Session):
             self.nextFunction()
             return
 
-        byebye_client_list = []
-        future_clients_exec_args = []
+        #byebye_client_list = []
+        #future_clients_exec_args = []
+
+
+
+        keep_client_list = [] # clients we will keep alive
+        byebye_client_list = [] # stopped clients we will remove immediately
 
         if not clear_all_clients:
             for future_client in self.future_clients:
-                if future_client.auto_start:
-                    future_clients_exec_args.append(
-                     (future_client.executable_path, future_client.arguments))
+                if not future_client.auto_start:
+                    continue
 
-        has_keep_alive = False
+                for client in self.clients:
+                    if client in keep_client_list:
+                        continue
+
+                    if client.canSwitchWith(future_client):
+                        client.switch_state = ray.SwitchState.RESERVED
+                        keep_client_list.append(client)
+                        break
 
         for client in self.clients:
-            if (not clear_all_clients
-                and (client.active and client.isCapableOf(':switch:')
-                     or (client.isDumbClient() and client.isRunning()))
-                and ((client.running_executable, client.running_arguments)
-                     in future_clients_exec_args)):
-                # client will switch
-                # or keep alive if non active and running
-                has_keep_alive = True
-                client.switch_state = ray.SwitchState.RESERVED
-                future_clients_exec_args.remove(
-                    (client.running_executable, client.running_arguments))
-            else:
+            if client not in keep_client_list:
                 # client is not capable of switch, or is not wanted
                 # in the new session
                 if client.isRunning():
@@ -1156,11 +1170,47 @@ class OperatingSession(Session):
                 else:
                     byebye_client_list.append(client)
 
-        if has_keep_alive:
+        if keep_client_list:
             self.setServerStatus(ray.ServerStatus.CLEAR)
         else:
             self.setServerStatus(ray.ServerStatus.CLOSE)
 
+
+
+
+        #if not clear_all_clients:
+            #for future_client in self.future_clients:
+                #if future_client.auto_start:
+                    #future_clients_exec_args.append(
+                     #(future_client.executable_path, future_client.arguments))
+
+        #has_keep_alive = False
+
+        #for client in self.clients:
+            #if (not clear_all_clients
+                #and (client.active and client.isCapableOf(':switch:')
+                     #or (client.isDumbClient() and client.isRunning()))
+                #and ((client.running_executable, client.running_arguments)
+                     #in future_clients_exec_args)):
+                ## client will switch
+                ## or keep alive if non active and running
+                #has_keep_alive = True
+                #client.switch_state = ray.SwitchState.RESERVED
+                #future_clients_exec_args.remove(
+                    #(client.running_executable, client.running_arguments))
+            #else:
+                ## client is not capable of switch, or is not wanted
+                ## in the new session
+                #if client.isRunning():
+                    #self.expected_clients.append(client)
+                #else:
+                    #byebye_client_list.append(client)
+
+        #if has_keep_alive:
+            #self.setServerStatus(ray.ServerStatus.CLEAR)
+        #else:
+            #self.setServerStatus(ray.ServerStatus.CLOSE)
+
         for client in byebye_client_list:
             if client in self.clients:
                 self.removeClient(client)
@@ -1206,7 +1256,9 @@ class OperatingSession(Session):
         self.cleanExpected()
         self.clients.clear()
         self.setPath('')
-        self.sendGui("/ray/gui/session/name", "", "")
+        self.sendGui('/ray/gui/session/name', '', '')
+        self.sendGui('/ray/gui/session/notes', '')
+        self.sendGui('/ray/gui/session/notes_hidden')
         self.noFuture()
         self.sendReply("Closed.")
         self.message("Done")
@@ -1217,7 +1269,9 @@ class OperatingSession(Session):
         self.cleanExpected()
         self.clients.clear()
         self.setPath('')
-        self.sendGui("/ray/gui/session/name", "", "")
+        self.sendGui('/ray/gui/session/name', '', '')
+        self.sendGui('/ray/gui/notes', '')
+        self.sendGui('/ray/gui/session/notes_hidden')
         self.noFuture()
         self.sendReply("Aborted.")
         self.message("Done")
@@ -1294,15 +1348,24 @@ for better organization.""")
                    ray.highlightText(new_session_full_name)))
 
         for client in self.clients:
-            client.net_duplicate_state = -1
+            if client.protocol == ray.Protocol.RAY_NET:
+                client.ray_net.duplicate_state = -1
+                if (client.ray_net.daemon_url
+                        and ray.isValidOscUrl(client.ray_net.daemon_url)):
+                    self.send(Address(client.ray_net.daemon_url),
+                              '/ray/session/duplicate_only',
+                              self.getShortPath(),
+                              new_session_full_name,
+                              client.ray_net.session_root)
+            #client.net_duplicate_state = -1
 
-            if (client.net_daemon_url
-                    and ray.isValidOscUrl(client.net_daemon_url)):
-                self.send(Address(client.net_daemon_url),
-                          '/ray/session/duplicate_only',
-                          self.getShortPath(),
-                          new_session_full_name,
-                          client.net_session_root)
+            #if (client.net_daemon_url
+                    #and ray.isValidOscUrl(client.net_daemon_url)):
+                #self.send(Address(client.net_daemon_url),
+                          #'/ray/session/duplicate_only',
+                          #self.getShortPath(),
+                          #new_session_full_name,
+                          #client.net_session_root)
 
                 self.expected_clients.append(client)
 
@@ -1332,7 +1395,8 @@ for better organization.""")
 
         self.sendGuiMessage(_translate('GUIMSG', '...session copy finished.'))
         for client in self.clients:
-            if 0 <= client.net_duplicate_state < 1:
+            if (client.protocol == ray.Protocol.RAY_NET
+                    and 0 <= client.ray_net.duplicate_state < 1):
                 self.expected_clients.append(client)
 
         if self.expected_clients:
@@ -1393,12 +1457,13 @@ for better organization.""")
         # on the machine where is the master daemon, for example).
 
         for client in self.clients:
-            if client.net_daemon_url:
-                self.send(Address(client.net_daemon_url),
+            if (client.protocol == ray.Protocol.RAY_NET
+                    and client.ray_net.daemon_url):
+                self.send(Address(client.ray_net.daemon_url),
                           '/ray/server/save_session_template',
-                          self.name,
+                          self.getShortPath(),
                           template_name,
-                          client.net_session_root)
+                          client.ray_net.session_root)
 
         self.setServerStatus(ray.ServerStatus.COPY)
 
@@ -1619,6 +1684,8 @@ for better organization."""))
                 return
 
             sess_name = content.attribute('name')
+            if content.attribute('notes_shown').lower() in ('1', 'true'):
+                self.future_notes_shown = True
 
             client_id_list = []
 
@@ -1651,8 +1718,7 @@ for better organization."""))
                         client_id_list.append(client.client_id)
 
                 elif tag_name == "Windows":
-                    server = self.getServer()
-                    if server and server.option_desktops_memory:
+                    if self.hasServerOption(ray.Option.DESKTOPS_MEMORY):
                         self.desktops_memory.readXml(node.toElement())
 
             ray_file.close()
@@ -1674,6 +1740,7 @@ for better organization."""))
                     client.prefix_mode = ray.PrefixMode.CLIENT_NAME
                     client.auto_start = True
                     client._from_nsm_file = True
+
                     self.future_clients.append(client)
 
             file.close()
@@ -1709,6 +1776,11 @@ for better organization."""))
 
         self.notes = self.future_notes
         self.sendGui('/ray/gui/session/notes', self.notes)
+        self.notes_shown = self.future_notes_shown
+        if self.notes_shown:
+            self.sendGui('/ray/gui/session/notes_shown')
+        else:
+            self.sendGui('/ray/gui/session/notes_hidden')
 
         self.load_locked = True
 
@@ -1730,7 +1802,7 @@ for better organization."""))
         for client in self.clients.__reversed__():
             if (open_off
                     or not client.isRunning()
-                    or client.isReplyPending()
+                    or (client.isReplyPending() and not client.isDumbClient())
                     or client.switch_state != ray.SwitchState.RESERVED):
                 self.clients_to_quit.append(client)
                 self.expected_clients.append(client)
@@ -1772,7 +1844,7 @@ for better organization."""))
         for i in rm_indexes:
             self.removeClient(self.clients[i])
 
-        # Lie to the GUI saying all clients are removed.
+        # Lie to the GUIs saying all clients are removed.
         # Clients will reappear just in a few time
         # It prevents GUI to have 2 clients with the same client_id
         # in the same time
@@ -1795,19 +1867,13 @@ for better organization."""))
                 for client in self.clients:
                     if (client.switch_state == ray.SwitchState.NEEDED
                             and client.client_id == future_client.client_id
-                            and client.running_executable
-                                    == future_client.executable_path
-                            and client.running_arguments
-                                    == future_client.arguments):
+                            and client.canSwitchWith(future_client)):
                         #we found the good existing client
                         break
                 else:
                     for client in self.clients:
                         if (client.switch_state == ray.SwitchState.NEEDED
-                                and client.running_executable
-                                    == future_client.executable_path
-                                and client.running_arguments
-                                    == future_client.arguments):
+                                and client.canSwitchWith(future_client)):
                             # we found a switchable client
                             break
                     else:
@@ -1898,8 +1964,7 @@ for better organization."""))
     def load_substep5(self):
         self.cleanExpected()
 
-        server = self.getServer()
-        if server and server.option_desktops_memory:
+        if self.hasServerOption(ray.Option.DESKTOPS_MEMORY):
             self.desktops_memory.replace()
 
         self.tellAllClientsSessionIsLoaded()
@@ -1911,6 +1976,15 @@ for better organization."""))
 
         self.switching_session = False
 
+        # display optional GUIs we want to be shown now
+        if self.hasServerOption(ray.Option.GUI_STATES):
+            for client in self.clients:
+                if (client.isRunning()
+                        and client.isCapableOf(':optional-gui:')
+                        and not client.start_gui_hidden
+                        and not client.gui_has_been_visible):
+                    client.sendToSelfAddress('/nsm/client/show_optional_gui')
+
         self.nextFunction()
 
     def loadDone(self):
@@ -1964,7 +2038,7 @@ for better organization.""")
         QCoreApplication.quit()
 
     def addClientTemplate(self, src_addr, src_path,
-                          template_name, factory=False):
+                          template_name, factory=False, auto_start=True):
         search_paths = self.getSearchTemplateDirs(factory)
 
         for search_path in search_paths:
@@ -2061,7 +2135,6 @@ for better organization.""")
                     if not program_version:
                         continue
 
-
                     neededs = []
                     progvss = []
 
@@ -2095,6 +2168,10 @@ for better organization.""")
                                 ray.Err.NOT_NOW)
                     return
 
+                client.template_origin = template_name
+                if not auto_start:
+                    client.auto_start = False
+
                 if full_name_files:
                     client.setStatus(ray.ClientStatus.PRECOPY)
                     self.file_copier.startClientCopy(
@@ -2184,6 +2261,10 @@ for better organization.""")
         client.start()
         self.nextFunction()
 
+    def loadClientSnapshotDone(self):
+        self.send(self.osc_src_addr, '/reply', self.osc_path,
+                  'Client snapshot loaded')
+
     def terminateStepScripter(self):
         if self.step_scripter.isRunning():
             self.step_scripter.terminate()
