@@ -3,6 +3,7 @@ import os
 import shutil
 import subprocess
 import sys
+import time
 from liblo import Address
 from PyQt5.QtCore import QCoreApplication, QProcess
 from PyQt5.QtXml  import QDomDocument
@@ -194,7 +195,7 @@ class SignaledSession(OperatingSession):
             server = self.getServer()
             if (server
                     and server.getServerStatus() == ray.ServerStatus.READY
-                    and server.option_desktops_memory):
+                    and server.options & ray.Option.DESKTOPS_MEMORY):
                 self.desktops_memory.replace()
         else:
             self.message("Reply from unknown client")
@@ -324,29 +325,32 @@ class SignaledSession(OperatingSession):
                 if not template_name or template_name in template_list:
                     continue
 
-                # check if needed executables are present
-                executable = ct.attribute('executable')
+                protocol = ray.protocolFromStr(ct.attribute('protocol'))
 
-                if not executable:
-                    continue
+                if protocol != ray.Protocol.RAY_NET:
+                    # check if needed executables are present
+                    executable = ct.attribute('executable')
 
-                try_exec_line = ct.attribute('try-exec')
+                    if not executable:
+                        continue
 
-                try_exec_list = []
-                if try_exec_line:
-                    try_exec_list = ct.attribute('try-exec').split(';')
+                    try_exec_line = ct.attribute('try-exec')
 
-                try_exec_list.append(executable)
-                try_exec_ok = True
+                    try_exec_list = []
+                    if try_exec_line:
+                        try_exec_list = ct.attribute('try-exec').split(';')
 
-                for try_exec in try_exec_list:
-                    exec_path = shutil.which(try_exec)
-                    if not exec_path:
-                        try_exec_ok = False
-                        break
+                    try_exec_list.append(executable)
+                    try_exec_ok = True
 
-                if not try_exec_ok:
-                    continue
+                    for try_exec in try_exec_list:
+                        exec_path = shutil.which(try_exec)
+                        if not exec_path:
+                            try_exec_ok = False
+                            break
+
+                    if not try_exec_ok:
+                        continue
 
                 # search for '/nsm/server/announce' in executable binary
                 # if it is asked by "check_nsm_bin" key
@@ -378,6 +382,10 @@ class SignaledSession(OperatingSession):
                             self.sendGui('/ray/gui/client_template_update',
                                         int(factory), template_name,
                                         *template_client.spread())
+                            if template_client.protocol == ray.Protocol.RAY_HACK:
+                                self.sendGui('/ray/gui/client_template_ray_hack_update',
+                                             int(factory), template_name,
+                                             *template_client.ray_hack.spread())
 
                     tmp_template_list.clear()
 
@@ -390,6 +398,11 @@ class SignaledSession(OperatingSession):
                     self.sendGui('/ray/gui/client_template_update',
                                 int(factory), template_name,
                                 *template_client.spread())
+                    if template_client.protocol == ray.Protocol.RAY_HACK:
+                        self.sendGui('/ray/gui/client_template_ray_hack_update',
+                                        int(factory), template_name,
+                                        *template_client.ray_hack.spread())
+
 
         # send a last empty reply to say list is finished
         self.send(src_addr, '/reply', path)
@@ -411,13 +424,16 @@ class SignaledSession(OperatingSession):
 
     def _ray_server_list_sessions(self, path, args, src_addr):
         with_net = False
+        last_sent_time = time.time()
+
         if args:
             with_net = args[0]
 
         if with_net:
             for client in self.clients:
-                if client.net_daemon_url:
-                    self.send(Address(client.net_daemon_url),
+                if (client.protocol == ray.Protocol.RAY_NET
+                        and client.ray_net.daemon_url):
+                    self.send(Address(client.ray_net.daemon_url),
                               '/ray/server/list_sessions', 1)
 
         if not self.root:
@@ -445,8 +461,10 @@ class SignaledSession(OperatingSession):
 
                     basefolder = root.replace(self.root + '/', '', 1)
                     session_list.append(basefolder)
+                    n += len(basefolder)
 
-                    if n >= 16000:
+                    if n >= 10000 or time.time() - last_sent_time > 0.300:
+                        last_sent_time = time.time()
                         self.send(src_addr, "/reply", path,
                                     *session_list)
 
@@ -489,22 +507,23 @@ class SignaledSession(OperatingSession):
 
             if not os.path.exists(spath):
                 self.steps_order = [self.save,
-                                      self.closeNoSaveClients,
-                                      self.snapshot,
-                                      (self.prepareTemplate, *args, False),
-                                      (self.preload, session_name),
-                                       self.takePlace,
-                                       self.load,
-                                       self.newDone]
+                                    self.closeNoSaveClients,
+                                    self.snapshot,
+                                    (self.prepareTemplate, *args, False),
+                                    (self.preload, session_name),
+                                    self.close,
+                                    self.takePlace,
+                                    self.load,
+                                    self.newDone]
                 return
 
         self.steps_order = [self.save,
-                              self.closeNoSaveClients,
-                              self.snapshot,
-                              self.close,
-                              (self.new, args[0]),
-                              self.save,
-                              self.newDone]
+                            self.closeNoSaveClients,
+                            self.snapshot,
+                            self.close,
+                            (self.new, args[0]),
+                            self.save,
+                            self.newDone]
 
     @session_operation
     def _ray_server_open_session(self, path, args, src_addr, open_off=False):
@@ -583,6 +602,36 @@ class SignaledSession(OperatingSession):
         tmp_session = DummySession(self.root)
         tmp_session.ray_server_rename_session(path, args, src_addr)
 
+    def _ray_server_save_session_template(self, path, args, src_addr):
+        if len(args) == 2:
+            session_name, template_name = args
+            sess_root = self.root
+            net = False
+        else:
+            session_name, template_name, sess_root = args
+            net = True
+
+        if (sess_root != self.root
+                or session_name != self.getShortPath()):
+            tmp_session = DummySession(sess_root)
+            tmp_session.ray_server_save_session_template(path,
+                                [session_name, template_name, net],
+                                src_addr)
+            return
+
+        self._ray_session_save_as_template(
+            path, [template_name, net], src_addr)
+
+    def _ray_server_set_option(self, path, args, src_addr):
+        option = args[0]
+
+        if abs(option) == ray.Option.BOOKMARK_SESSION:
+            if self.path:
+                if option > 0:
+                    self.bookmarker.makeAll(self.path)
+                else:
+                    self.bookmarker.removeAll(self.path)
+
     @session_operation
     def _ray_session_save(self, path, args, src_addr):
         self.steps_order = [self.save, self.snapshot, self.saveDone]
@@ -593,47 +642,12 @@ class SignaledSession(OperatingSession):
         net = False if len(args) < 2 else args[1]
 
         for client in self.clients:
-            if client.executable_path == 'ray-network':
-                client.net_session_template = template_name
+            if client.protocol == ray.Protocol.RAY_NET:
+                client.ray_net.session_template = template_name
 
         self.steps_order = [self.save, self.snapshot,
-                              (self.saveSessionTemplate,
-                               template_name, net)]
-
-    def _ray_server_save_session_template(self, path, args, src_addr):
-        if len(args) == 2:
-            session_name, template_name = args
-            sess_root = self.root
-            net = False
-        else:
-            session_name, template_name, sess_root = args
-            net = True
-
-        tmp_session = DummySession(sess_root)
-        tmp_session.ray_server_save_session_template(path,
-                                [session_name, template_name, net],
-                                src_addr)
-
-        #if (sess_root != self.root or session_name != self.name):
-            #tmp_session = DummySession(sess_root)
-            #tmp_session.ray_server_save_session_template(path,
-                                #[session_name, template_name, net],
-                                #src_addr)
-            #return
-
-        #self.ray_session_save_as_template(path, [template_name, net],
-                                          #src_addr)
-
-
-        #if net:
-            #for client in self.clients:
-                #if client.executable_path == 'ray-network':
-                    #client.net_session_template = template_name
-
-        #self.rememberOscArgs()
-        #self.steps_order = [self.save, self.snapshot,
-                              #(self.saveSessionTemplate, template_name, net)]
-        #self.nextFunction()
+                            (self.saveSessionTemplate,
+                             template_name, net)]
 
     @session_operation
     def _ray_session_take_snapshot(self, path, args, src_addr):
@@ -988,14 +1002,15 @@ class SignaledSession(OperatingSession):
 
         factory = bool(args[0])
         template_name = args[1]
+        auto_start = bool(len(args) <= 2 or args[2] != 'not_start')
 
-        self.addClientTemplate(src_addr, path, template_name, factory)
+        self.addClientTemplate(src_addr, path, template_name, factory, auto_start)
 
     def _ray_session_add_factory_client_template(self, path, args, src_addr):
-        self._ray_session_add_client_template(path, [1, args[0]], src_addr)
+        self._ray_session_add_client_template(path, [1] + args, src_addr)
 
     def _ray_session_add_user_client_template(self, path, args, src_addr):
-        self._ray_session_add_client_template(path, [0, args[0]], src_addr)
+        self._ray_session_add_client_template(path, [0] + args, src_addr)
 
     def _ray_session_reorder_clients(self, path, args, src_addr):
         client_ids_list = args
@@ -1227,6 +1242,7 @@ class SignaledSession(OperatingSession):
     @client_action
     def _ray_client_show_optional_gui(self, path, args, src_addr, client):
         client.sendToSelfAddress("/nsm/client/show_optional_gui")
+        client.show_gui_ordered = True
         self.send(src_addr, '/reply', path, 'show optional GUI asked')
 
     @client_action
@@ -1258,6 +1274,13 @@ class SignaledSession(OperatingSession):
 
         self.send(src_addr, '/reply', path, 'ray_hack updated')
 
+    @client_action
+    def _ray_client_update_ray_net_properties(self, path, args,
+                                              src_addr, client):
+        if client.protocol == ray.Protocol.RAY_NET:
+            client.ray_net.update(*args)
+        self.send(src_addr, '/reply', path, 'ray_net updated')
+
     @client_action
     def _ray_client_set_properties(self, path, args, src_addr, client):
         message = ''
@@ -1308,7 +1331,7 @@ class SignaledSession(OperatingSession):
                      'save_signal', 'stop_signal',
                      'no_save_level', 'wait_window',
                      'VERSION'):
-            message += "%s:%s\n" % (prop, cte.attribute(property))
+            message += "%s:%s\n" % (prop, cte.attribute(prop))
 
         # remove last empty line
         message = message.rpartition('\n')[0]
@@ -1433,12 +1456,14 @@ class SignaledSession(OperatingSession):
                         (self.snapshot, '', snapshot, True),
                         (self.closeClient, client),
                         (self.loadClientSnapshot, client_id, snapshot),
-                        (self.startClient, client)]
+                        (self.startClient, client),
+                        self.loadClientSnapshotDone]
                 else:
                     self.steps_order = [
                         self.save,
                         (self.snapshot, '', snapshot, True),
-                        (self.loadClientSnapshot, client_id, snapshot)]
+                        (self.loadClientSnapshot, client_id, snapshot),
+                        self.loadClientSnapshotDone]
                 break
         else:
             self.sendErrorNoClient(src_addr, path, client_id)
@@ -1565,10 +1590,11 @@ class SignaledSession(OperatingSession):
     def _ray_net_daemon_duplicate_state(self, path, args, src_addr):
         state = args[0]
         for client in self.clients:
-            if (client.net_daemon_url
-                    and ray.areSameOscPort(client.net_daemon_url,
+            if (client.protocol == ray.Protocol.RAY_NET
+                    and client.ray_net.daemon_url
+                    and ray.areSameOscPort(client.ray_net.daemon_url,
                                            src_addr.url)):
-                client.net_duplicate_state = state
+                client.ray_net.duplicate_state = state
                 client.net_daemon_copy_timer.stop()
                 break
         else:
@@ -1584,13 +1610,6 @@ class SignaledSession(OperatingSession):
 
         client.net_daemon_copy_timer.start()
 
-    def _ray_option_bookmark_session_folder(self, path, args, src_addr):
-        if self.path:
-            if args[0]:
-                self.bookmarker.makeAll(self.path)
-            else:
-                self.bookmarker.removeAll(self.path)
-
     def serverOpenSessionAtStart(self, session_name):
         self.steps_order = [(self.preload, session_name),
                             self.takePlace,
