@@ -3,15 +3,18 @@ from PyQt5.QtWidgets import (QDialogButtonBox, QListWidgetItem, QFrame,
                              QMenu, QAction)
 from PyQt5.QtGui import QIcon, QPalette
 
+import client_properties_dialog
 import ray
 
-from gui_tools import RS, _translate
+from gui_tools import RS, _translate, isDarkTheme
 from child_dialogs import ChildDialog
+from gui_signaler import Signaler
 
 import ui_add_application
 import ui_template_slot
 import ui_remove_template
 
+
 class TemplateSlot(QFrame):
     def __init__(self, list_widget, item, session,
                  name, factory, client_data):
@@ -41,50 +44,39 @@ class TemplateSlot(QFrame):
 
         self.is_favorite = False
 
-        self.favicon_not = QIcon(':scalable/breeze/draw-star.svg')
+        self.ui.toolButtonFavorite.setSession(self._session)
+        self.ui.toolButtonFavorite.setTemplate(
+            name, self.client_data.icon, self.factory)
 
-        if (self.palette().brush(2, QPalette.WindowText).color().lightness()
-                > 128):
+        if isDarkTheme(self):
             self.ui.toolButtonUser.setIcon(
                 QIcon(':scalable/breeze-dark/im-user.svg'))
-            self.favicon_not = QIcon(':scalable/breeze-dark/draw-star.svg')
-
-        for favorite in self._session.favorite_list:
-            if favorite.name == name and favorite.factory == factory:
-                self.is_favorite = True
-                break
-
-        if self.is_favorite:
-            self.ui.toolButtonFavorite.setIcon(
-                QIcon(':scalable/breeze/star-yellow.svg'))
-        else:
-            self.ui.toolButtonFavorite.setIcon(self.favicon_not)
-
-        self.ui.toolButtonFavorite.clicked.connect(self.favoriteClicked)
+            self.ui.toolButtonFavorite.setDarkTheme()
 
     def updateClientData(self, *args):
         self.client_data.update(*args)
         self.ui.toolButtonIcon.setIcon(
             ray.getAppIcon(self.client_data.icon, self))
+        self.ui.toolButtonFavorite.setTemplate(
+            self.name, self.client_data.icon, self.factory)
 
-    def favoriteClicked(self):
-        self.is_favorite = not self.is_favorite
+    def updateRayHackData(self, *args):
+        if self.client_data.ray_hack is None:
+            self.client_data.ray_hack = ray.RayHack.newFrom(*args)
+        self.client_data.ray_hack.update(*args)
 
-        if self.is_favorite:
-            self.ui.toolButtonFavorite.setIcon(
-                QIcon(':scalable/breeze/star-yellow.svg'))
-            self._session.addFavorite(self.name, self.client_data.icon,
-                                      self.factory)
-        else:
-            self.ui.toolButtonFavorite.setIcon(self.favicon_not)
-            self._session.removeFavorite(self.name, self.factory)
-
-        self.list_widget.setCurrentItem(self.item)
+    def updateRayNetData(self, *args):
+        if self.client_data.ray_net is None:
+            self.client_data.ray_net = ray.RayNet.newFrom(*args)
+        self.client_data.ray_net.update(*args)
 
     def removeTemplate(self):
         add_app_dialog = self.list_widget.parent()
         add_app_dialog.removeTemplate(self.name, self.factory)
 
+    def setAsFavorite(self, bool_favorite):
+        self.ui.toolButtonFavorite.setAsFavorite(bool_favorite)
+
     def mouseDoubleClickEvent(self, event):
         self.list_widget.parent().accept()
 
@@ -122,6 +114,12 @@ class TemplateItem(QListWidgetItem):
     def updateClientData(self, *args):
         self.f_widget.updateClientData(*args)
 
+    def updateRayHackData(self, *args):
+        self.f_widget.updateRayHackData(*args)
+
+    def setAsFavorite(self, bool_favorite: bool):
+        self.f_widget.setAsFavorite(bool_favorite)
+
 
 class RemoveTemplateDialog(ChildDialog):
     def __init__(self, parent, template_name):
@@ -148,9 +146,14 @@ class AddApplicationDialog(ChildDialog):
             'AddApplication/factory_box', True, type=bool))
         self.ui.checkBoxUser.setChecked(RS.settings.value(
             'AddApplication/user_box', True, type=bool))
+        self.ui.checkBoxRayHack.setChecked(RS.settings.value(
+            'AddApplication/ray_hack_box', True, type=bool))
+        self.ui.widgetTemplateInfos.setVisible(False)
 
         self.ui.checkBoxFactory.stateChanged.connect(self.factoryBoxChanged)
         self.ui.checkBoxUser.stateChanged.connect(self.userBoxChanged)
+        self.ui.checkBoxNsm.stateChanged.connect(self.nsmBoxChanged)
+        self.ui.checkBoxRayHack.stateChanged.connect(self.rayHackBoxChanged)
 
         self.ui.templateList.currentItemChanged.connect(
             self.currentItemChanged)
@@ -159,14 +162,40 @@ class AddApplicationDialog(ChildDialog):
         self.ui.filterBar.updownpressed.connect(self.updownPressed)
         self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
 
+        self.user_menu = QMenu()
+        act_remove_template = QAction(QIcon.fromTheme('edit-delete-remove'),
+                                      _translate('menu', 'remove'),
+                                      self.user_menu)
+        act_remove_template.triggered.connect(self.removeCurrentTemplate)
+        self.user_menu.addAction(act_remove_template)
+        self.ui.toolButtonUser.setMenu(self.user_menu)
+        self.ui.toolButtonFavorite.setSession(self._session)
+        self.ui.widgetNonSaveable.setVisible(False)
+        self.ui.toolButtonAdvanced.clicked.connect(self.toolButtonAdvancedClicked)
+
+        if isDarkTheme(self):
+            self.ui.toolButtonUser.setIcon(
+                QIcon(':scalable/breeze-dark/im-user.svg'))
+            self.ui.toolButtonFavorite.setDarkTheme()
+            self.ui.toolButtonNoSave.setIcon(
+                QIcon(':scalable/breeze-dark/document-nosave.svg'))
+
         self._signaler.user_client_template_found.connect(
             self.addUserTemplates)
         self._signaler.factory_client_template_found.connect(
             self.addFactoryTemplates)
         self._signaler.client_template_update.connect(
             self.updateClientTemplate)
+        self._signaler.client_template_ray_hack_update.connect(
+            self.updateClientTemplateRayHack)
+        self._signaler.client_template_ray_net_update.connect(
+            self.updateClientTemplateRayNet)
+        self._signaler.favorite_added.connect(self.favoriteAdded)
+        self._signaler.favorite_removed.connect(self.favoriteRemoved)
+
         self.toDaemon('/ray/server/list_user_client_templates')
         self.toDaemon('/ray/server/list_factory_client_templates')
+        self.listing_finished = 0
 
         self.user_template_list = []
         self.factory_template_list = []
@@ -176,6 +205,35 @@ class AddApplicationDialog(ChildDialog):
 
         self.serverStatusChanged(self._session.server_status)
 
+        self.ui.filterBar.setFocus()
+
+    def favoriteAdded(self, template_name: str,
+                      template_icon: str, factory: bool):
+        for i in range(self.ui.templateList.count()):
+            item = self.ui.templateList.item(i)
+            if item is None:
+                continue
+
+            if (item.data(Qt.UserRole) == template_name
+                    and item.f_factory == factory):
+                item.setAsFavorite(True)
+                if item == self.ui.templateList.currentItem():
+                    self.ui.toolButtonFavorite.setAsFavorite(True)
+                break
+
+    def favoriteRemoved(self, template_name: str, factory: bool):
+        for i in range(self.ui.templateList.count()):
+            item = self.ui.templateList.item(i)
+            if item is None:
+                continue
+
+            if (item.data(Qt.UserRole) == template_name
+                    and item.f_factory == factory):
+                item.setAsFavorite(False)
+                if item == self.ui.templateList.currentItem():
+                    self.ui.toolButtonFavorite.setAsFavorite(False)
+                break
+
     def factoryBoxChanged(self, state):
         if not state:
             self.ui.checkBoxUser.setChecked(True)
@@ -188,6 +246,18 @@ class AddApplicationDialog(ChildDialog):
 
         self.updateFilteredList()
 
+    def nsmBoxChanged(self, state):
+        if not state:
+            self.ui.checkBoxRayHack.setChecked(True)
+
+        self.updateFilteredList()
+
+    def rayHackBoxChanged(self, state):
+        if not state:
+            self.ui.checkBoxNsm.setChecked(True)
+
+        self.updateFilteredList()
+
     def serverStatusChanged(self, server_status):
         self.server_will_accept = bool(
             server_status not in (
@@ -213,7 +283,8 @@ class AddApplicationDialog(ChildDialog):
                                        icon_name,
                                        template_name,
                                        False)
-
+            if self._session.isFavorite(template_name, False):
+                list_widget.setAsFavorite(True)
             self.ui.templateList.addItem(list_widget)
 
             self.ui.templateList.sortItems()
@@ -238,7 +309,8 @@ class AddApplicationDialog(ChildDialog):
                                        icon_name,
                                        template_name,
                                        True)
-
+            if self._session.isFavorite(template_name, True):
+                list_widget.setAsFavorite(True)
             self.ui.templateList.addItem(list_widget)
             self.ui.templateList.sortItems()
 
@@ -253,6 +325,34 @@ class AddApplicationDialog(ChildDialog):
 
             if item.matchesWith(factory, template_name):
                 item.updateClientData(*args[2:])
+                if self.ui.templateList.currentItem() == item:
+                    self.updateTemplateInfos(item)
+                break
+
+    def updateClientTemplateRayHack(self, args):
+        factory = bool(args[0])
+        template_name = args[1]
+
+        for i in range(self.ui.templateList.count()):
+            item = self.ui.templateList.item(i)
+
+            if item.matchesWith(factory, template_name):
+                item.updateRayHackData(*args[2:])
+                if self.ui.templateList.currentItem() == item:
+                    self.updateTemplateInfos(item)
+                break
+
+    def updateClientTemplateRayNet(self, args):
+        factory = bool(args[0])
+        template_name = args[1]
+
+        for i in range(self.ui.templateList.count()):
+            item = self.ui.templateList.item(i)
+
+            if item.matchesWith(factory, template_name):
+                item.updateRayNetData(*args[2:])
+                if self.ui.templateList.currentItem() == item:
+                    self.updateTemplateInfos(item)
                 break
 
     def updateFilteredList(self, filt=''):
@@ -276,6 +376,15 @@ class AddApplicationDialog(ChildDialog):
             if not item.f_factory and not self.ui.checkBoxUser.isChecked():
                 item.setHidden(True)
 
+            if item.client_data is not None:
+                if (item.client_data.protocol == ray.Protocol.RAY_HACK
+                        and not self.ui.checkBoxRayHack.isChecked()):
+                    item.setHidden(True)
+
+                if (item.client_data.protocol != ray.Protocol.RAY_HACK
+                        and not self.ui.checkBoxNsm.isChecked()):
+                    item.setHidden(True)
+
         # if selected item not in list, then select the first visible
         if (not self.ui.templateList.currentItem()
                 or self.ui.templateList.currentItem().isHidden()):
@@ -286,8 +395,9 @@ class AddApplicationDialog(ChildDialog):
 
         if (not self.ui.templateList.currentItem()
                 or self.ui.templateList.currentItem().isHidden()):
-            self.ui.filterBar.setStyleSheet(
-                "QLineEdit { background-color: red}")
+            if self.ui.filterBar.text():
+                self.ui.filterBar.setStyleSheet(
+                    "QLineEdit { background-color: red}")
             self.ui.templateList.setCurrentItem(None)
         else:
             self.ui.filterBar.setStyleSheet("")
@@ -313,10 +423,72 @@ class AddApplicationDialog(ChildDialog):
                 row += 1
         self.ui.templateList.setCurrentRow(row)
 
+    def updateTemplateInfos(self, item):
+        self.ui.widgetTemplateInfos.setVisible(bool(item))
+        self.ui.widgetNoTemplate.setVisible(not bool(item))
+
+        if not item:
+            return
+
+        cdata = item.client_data
+        self.ui.toolButtonIcon.setIcon(
+            ray.getAppIcon(cdata.icon, self))
+        self.ui.labelTemplateName.setText(item.data(Qt.UserRole))
+        self.ui.labelDescription.setText(cdata.description)
+        self.ui.labelProtocol.setText(ray.protocolToStr(cdata.protocol))
+        self.ui.labelExecutable.setText(cdata.executable_path)
+        self.ui.labelLabel.setText(cdata.label)
+        self.ui.labelName.setText(cdata.name)
+
+        for widget in (self.ui.labelProtocolTitle,
+                       self.ui.labelProtocolColon,
+                       self.ui.labelProtocol):
+            widget.setVisible(bool(cdata.protocol != ray.Protocol.NSM))
+
+        for widget in (self.ui.labelLabelTitle,
+                        self.ui.labelLabelColon,
+                        self.ui.labelLabel):
+            widget.setVisible(bool(cdata.label))
+
+        for widget in (self.ui.labelNameTitle,
+                        self.ui.labelNameColon,
+                        self.ui.labelName):
+            widget.setVisible(bool(cdata.protocol == ray.Protocol.NSM))
+
+        self.ui.toolButtonUser.setVisible(not item.f_factory)
+        self.ui.toolButtonFavorite.setTemplate(
+            item.data(Qt.UserRole), cdata.icon, item.f_factory)
+        self.ui.toolButtonFavorite.setAsFavorite(self._session.isFavorite(
+            item.data(Qt.UserRole), item.f_factory))
+
+        self.ui.widgetNonSaveable.setVisible(bool(
+            cdata.ray_hack is not None
+            and cdata.protocol == ray.Protocol.RAY_HACK
+            and cdata.ray_hack.no_save_level > 0))
+
+        # little security
+        # client_properties_dialog could crash if ray_hack has not been updated yet
+        # (never seen this appears, but it could with slow systems)
+        self.ui.toolButtonAdvanced.setEnabled(
+            bool(cdata.protocol != ray.Protocol.RAY_HACK
+                 or cdata.ray_hack is not None))
+
     def currentItemChanged(self, item, previous_item):
         self.has_selection = bool(item)
+        self.updateTemplateInfos(item)
         self.preventOk()
 
+    def toolButtonAdvancedClicked(self):
+        item = self.ui.templateList.currentItem()
+        if item is None:
+            return
+
+        properties_dialog = client_properties_dialog.ClientPropertiesDialog.create(
+            self, item.client_data)
+        properties_dialog.updateContents()
+        properties_dialog.setForTemplate(item.data(Qt.UserRole))
+        properties_dialog.show()
+
     def preventOk(self):
         self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(
             bool(self.server_will_accept and self.has_selection))
@@ -339,9 +511,18 @@ class AddApplicationDialog(ChildDialog):
 
             if not item.f_factory and template_name == item.data(Qt.UserRole):
                 item.setHidden(True)
+                if item == self.ui.templateList.currentItem():
+                    self.updateTemplateInfos(None)
                 self.ui.templateList.removeItemWidget(item)
                 break
 
+    def removeCurrentTemplate(self):
+        item = self.ui.templateList.currentItem()
+        if not item:
+            return
+
+        self.removeTemplate(item.data(Qt.UserRole), False)
+
     def saveCheckBoxes(self):
         RS.settings.setValue(
             'AddApplication/factory_box',
@@ -349,4 +530,7 @@ class AddApplicationDialog(ChildDialog):
         RS.settings.setValue(
             'AddApplication/user_box',
             self.ui.checkBoxUser.isChecked())
+        RS.settings.setValue(
+            'AddApplication/ray_hack_box',
+            self.ui.checkBoxRayHack.isChecked())
         RS.settings.sync()
