@@ -3,11 +3,12 @@ import sys
 import time
 
 from PyQt5.QtWidgets import (
-    QDialog, QDialogButtonBox, QTreeWidgetItem,
+    QDialog, QDialogButtonBox, QTreeWidget, QTreeWidgetItem,
     QCompleter, QMessageBox, QFileDialog)
 from PyQt5.QtGui import QIcon, QPixmap, QGuiApplication
 from PyQt5.QtCore import Qt, QTimer
 
+import client_properties_dialog
 import ray
 from gui_server_thread import GUIServerThread
 from gui_tools import (ErrDaemon, _translate,
@@ -26,6 +27,7 @@ import ui_error_dialog
 import ui_quit_app
 import ui_script_info
 import ui_script_user_action
+import ui_session_notes
 import ui_session_scripts_info
 import ui_stop_client
 import ui_stop_client_no_save
@@ -34,6 +36,7 @@ import ui_client_trash
 import ui_daemon_url
 import ui_snapshot_progress
 import ui_waiting_close_user
+import ui_client_rename
 
 class ChildDialog(QDialog):
     def __init__(self, parent):
@@ -46,8 +49,16 @@ class ChildDialog(QDialog):
         self._signaler.server_status_changed.connect(self.serverStatusChanged)
         self._signaler.server_copying.connect(self.serverCopying)
 
+        self.root_folder_file_dialog = None
+        self.root_folder_message_box = QMessageBox(
+            QMessageBox.Critical,
+            _translate('root_folder_dialogs', 'unwritable dir'),
+            '',
+            QMessageBox.NoButton,
+            self)
+
         self.server_copying = parent.server_copying
-    
+
     @classmethod
     def toDaemon(cls, *args):
         server = GUIServerThread.instance()
@@ -65,38 +76,47 @@ class ChildDialog(QDialog):
         self.serverStatusChanged(self._session.server_status)
 
     def changeRootFolder(self):
-        root_folder = QFileDialog.getExistingDirectory(
-            self,
-            _translate("root_folder_dialogs",
-                       "Choose root folder for sessions"),
-            CommandLineArgs.session_root,
-            QFileDialog.ShowDirsOnly)
+        # construct this here only because it can be quite long
+        if self.root_folder_file_dialog is None:
+            self.root_folder_file_dialog = QFileDialog(
+                self,
+                _translate("root_folder_dialogs",
+                        "Choose root folder for sessions"),
+                CommandLineArgs.session_root)
+            self.root_folder_file_dialog.setFileMode(QFileDialog.Directory)
+            self.root_folder_file_dialog.setOption(QFileDialog.ShowDirsOnly)
+        else:
+            self.root_folder_file_dialog.setDirectory(CommandLineArgs.session_root)
 
-        if not root_folder:
+        self.root_folder_file_dialog.exec()
+        if not self.root_folder_file_dialog.result():
             return
 
+        selected_files = self.root_folder_file_dialog.selectedFiles()
+        if not selected_files:
+            return
+
+        root_folder = selected_files[0]
+
         # Security, kde dialogs sends $HOME if user type a folder path
         # that doesn't already exists.
         if os.getenv('HOME') and root_folder == os.getenv('HOME'):
             return
 
-        errorDialog = QMessageBox(
-            QMessageBox.Critical,
-            _translate('root_folder_dialogs', 'unwritable dir'),
-            _translate(
-                'root_folder_dialogs',
-                '<p>You have no permissions for %s,<br>' % root_folder \
-                    + 'choose another directory !</p>'))
+        self.root_folder_message_box.setText(
+            _translate('root_folder_dialogs',
+                "<p>You have no permissions for %s,<br>choose another directory !</p>")
+                    % root_folder)
 
         if not os.path.exists(root_folder):
             try:
                 os.makedirs(root_folder)
             except:
-                errorDialog.exec()
+                self.root_folder_message_box.exec()
                 return
 
         if not os.access(root_folder, os.W_OK):
-            errorDialog.exec()
+            self.root_folder_message_box.exec()
             return
 
         RS.settings.setValue('default_session_root', root_folder)
@@ -116,7 +136,7 @@ class SessionItem(QTreeWidgetItem):
     def __init__(self, l_list):
         QTreeWidgetItem.__init__(self, l_list)
 
-    def showConditionnaly(self, string):
+    def showConditionnaly(self, string: str)->bool:
         show = bool(string.lower() in self.data(0, Qt.UserRole).lower())
 
         n = 0
@@ -186,6 +206,14 @@ class OpenSessionDialog(ChildDialog):
         self.ui = ui_open_session.Ui_DialogOpenSession()
         self.ui.setupUi(self)
 
+        self.timer_progress = QTimer()
+        self.timer_progress.setInterval(50)
+        self.timer_progress_n = 0
+        self.timer_progress.timeout.connect(self.timerProgress)
+        self.timer_progress.start()
+        self.progress_inverted = False
+        self.ui.widgetSpacer.setVisible(False)
+
         self.ui.toolButtonFolder.clicked.connect(self.changeRootFolder)
         self.ui.sessionList.currentItemChanged.connect(
             self.currentItemChanged)
@@ -193,7 +221,8 @@ class OpenSessionDialog(ChildDialog):
         self.ui.sessionList.itemDoubleClicked.connect(self.goIfAny)
         self.ui.sessionList.itemClicked.connect(self.deployItem)
         self.ui.filterBar.textEdited.connect(self.updateFilteredList)
-        self.ui.filterBar.updownpressed.connect(self.updownPressed)
+        #self.ui.filterBar.updownpressed.connect(self.updownPressed)
+        self.ui.filterBar.key_event.connect(self.updownPressed)
         self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
         self.ui.currentSessionsFolder.setText(CommandLineArgs.session_root)
 
@@ -216,6 +245,9 @@ class OpenSessionDialog(ChildDialog):
         self.all_items = []
 
         self._last_mouse_click = 0
+        self._last_session_item = None
+
+        self.ui.filterBar.setFocus(Qt.OtherFocusReason)
 
     def serverStatusChanged(self, server_status):
         self.ui.toolButtonFolder.setEnabled(
@@ -227,8 +259,23 @@ class OpenSessionDialog(ChildDialog):
             server_status in (
                 ray.ServerStatus.OFF,
                 ray.ServerStatus.READY) and not self.server_copying)
+
+        if server_status != ray.ServerStatus.OFF:
+            if self.root_folder_file_dialog is not None:
+                self.root_folder_file_dialog.reject()
+            self.root_folder_message_box.reject()
+
         self.preventOk()
 
+    def timerProgress(self):
+        self.ui.progressBar.setValue(self.timer_progress_n)
+        if self.timer_progress_n >= 100:
+            self.timer_progress_n = 0
+            self.progress_inverted = not self.progress_inverted
+            self.ui.progressBar.setInvertedAppearance(
+                self.progress_inverted)
+        self.timer_progress_n += 5
+
     def rootChanged(self, session_root):
         self.ui.currentSessionsFolder.setText(session_root)
         self.ui.sessionList.clear()
@@ -236,6 +283,26 @@ class OpenSessionDialog(ChildDialog):
         self.toDaemon('/ray/server/list_sessions', 0)
 
     def addSessions(self, session_names):
+        if not session_names:
+            self.timer_progress.stop()
+            height = self.ui.progressBar.size().height()
+            self.ui.progressBar.setVisible(False)
+            self.ui.widgetSpacer.setVisible(True)
+
+            # Try to select last used session
+            root_item = self.ui.sessionList.invisibleRootItem()
+            for i in range(root_item.childCount()):
+                item = root_item.child(i)
+                last_session_item = item.findItemWith(
+                    RS.settings.value('last_session', type=str))
+
+                if last_session_item:
+                    self.ui.sessionList.setCurrentItem(last_session_item)
+                    self.ui.sessionList.scrollToItem(last_session_item)
+                    break
+            return
+
+
         for session_name in session_names:
             folder_div = session_name.split('/')
             folders = self.folders
@@ -264,18 +331,6 @@ class OpenSessionDialog(ChildDialog):
 
         self.ui.sessionList.sortByColumn(0, Qt.AscendingOrder)
 
-        # Try to select last used session
-        root_item = self.ui.sessionList.invisibleRootItem()
-        for i in range(root_item.childCount()):
-            item = root_item.child(i)
-            last_session_item = item.findItemWith(
-                RS.settings.value('last_session', type=str))
-
-            if last_session_item:
-                self.ui.sessionList.setCurrentItem(last_session_item)
-                self.ui.sessionList.scrollToItem(last_session_item)
-                break
-
     def updateFilteredList(self, filt):
         filter_text = self.ui.filterBar.displayText()
         root_item = self.ui.sessionList.invisibleRootItem()
@@ -302,28 +357,25 @@ class OpenSessionDialog(ChildDialog):
             self.ui.filterBar.setStyleSheet("")
             self.ui.sessionList.scrollTo(self.ui.sessionList.currentIndex())
 
-    def updownPressed(self, key):
-        root_item = self.ui.sessionList.invisibleRootItem()
-        row = self.ui.sessionList.currentIndex().row()
+    def updownPressed(self, event):
+        start_item = self.ui.sessionList.currentItem()
+        QTreeWidget.keyPressEvent(self.ui.sessionList, event)
+        if not start_item:
+            return
 
-        if key == Qt.Key_Up:
-            if row == 0:
-                return
-            row -= 1
-            while root_item.child(row).isHidden():
-                if row == 0:
-                    return
-                row -= 1
-        elif key == Qt.Key_Down:
-            if row == root_item.childCount() - 1:
-                return
-            row += 1
-            while root_item.child(row).isHidden():
-                if row == root_item.childCount() - 1:
-                    return
-                row += 1
+        current_item = self.ui.sessionList.currentItem()
+        if current_item == start_item:
+            return
 
-        self.ui.sessionList.setCurrentItem(root_item.child(row))
+        ex_item = current_item
+
+        while not current_item.flags() & Qt.ItemIsSelectable:
+            ex_item = current_item
+            QTreeWidget.keyPressEvent(self.ui.sessionList, event)
+            current_item = self.ui.sessionList.currentItem()
+            if current_item == ex_item:
+                self.ui.sessionList.setCurrentItem(start_item)
+                return
 
     def currentItemChanged(self, item, previous_item):
         self.has_selection = bool(item and item.data(0, Qt.UserRole))
@@ -417,6 +469,11 @@ class NewSessionDialog(ChildDialog):
             self.server_will_accept = bool(
                 server_status == ray.ServerStatus.READY and not self.server_copying)
 
+        if server_status != ray.ServerStatus.OFF:
+            if self.root_folder_file_dialog is not None:
+                self.root_folder_file_dialog.reject()
+            self.root_folder_message_box.reject()
+
         self.preventOk()
 
     def rootChanged(self, session_root):
@@ -534,13 +591,29 @@ class AbstractSaveTemplateDialog(ChildDialog):
 
         self.server_will_accept = False
 
+        self.update_template_text = _translate(
+            "session template", "Update the template")
+        self.create_template_text = self.ui.pushButtonAccept.text()
         self.ui.lineEdit.textEdited.connect(self.textEdited)
         self.ui.pushButtonAccept.clicked.connect(self.verifyAndAccept)
         self.ui.pushButtonAccept.setEnabled(False)
 
+        self.overwrite_message_box = QMessageBox(
+            QMessageBox.Question,
+            _translate(
+                    'session template',
+                    'Overwrite Template ?'),
+            '',
+            QMessageBox.Yes | QMessageBox.No,
+            self)
+
     def textEdited(self, text):
         if '/' in text:
             self.ui.lineEdit.setText(text.replace('/', '⁄'))
+        if self.ui.lineEdit.text() in self.template_list:
+            self.ui.pushButtonAccept.setText(self.update_template_text)
+        else:
+            self.ui.pushButtonAccept.setText(self.create_template_text)
         self.allowOkButton()
 
     def getTemplateName(self):
@@ -553,24 +626,32 @@ class AbstractSaveTemplateDialog(ChildDialog):
     def verifyAndAccept(self):
         template_name = self.getTemplateName()
         if template_name in self.template_list:
-            ret = QMessageBox.question(
-                self,
-                _translate(
-                    'session template',
-                    'Overwrite Template ?'),
+            self.overwrite_message_box.setText(
                 _translate(
                     'session_template',
                     'Template <strong>%s</strong> already exists.\nOverwrite it ?') %
                 template_name)
-            if ret == QMessageBox.No:
+
+            self.overwrite_message_box.exec()
+
+            if (self.overwrite_message_box.clickedButton()
+                    == self.overwrite_message_box.button(QMessageBox.No)):
                 return
         self.accept()
 
+    def addTemplatesToList(self, template_list):
+        self.template_list += template_list
+
+        for template in template_list:
+            if template == self.ui.lineEdit.text():
+                self.ui.pushButtonAccept.setText(self.update_template_text)
+                break
 
 class SaveTemplateSessionDialog(AbstractSaveTemplateDialog):
     def __init__(self, parent):
         AbstractSaveTemplateDialog.__init__(self, parent)
-
+        self.ui.toolButtonClientIcon.setVisible(False)
+        self.ui.labelLabel.setText(self._session.path)
         self.template_list = []
 
         self._signaler.session_template_found.connect(self.addTemplatesToList)
@@ -582,17 +663,19 @@ class SaveTemplateSessionDialog(AbstractSaveTemplateDialog):
         self.server_will_accept = bool(server_status == ray.ServerStatus.READY)
 
         if server_status == ray.ServerStatus.OFF:
+            self.overwrite_message_box.reject()
             self.reject()
 
         self.allowOkButton()
 
-    def addTemplatesToList(self, template_list):
-        self.template_list += template_list
-
 
 class SaveTemplateClientDialog(AbstractSaveTemplateDialog):
-    def __init__(self, parent):
+    def __init__(self, parent, client):
         AbstractSaveTemplateDialog.__init__(self, parent)
+        self.ui.labelSessionTitle.setVisible(False)
+        self.ui.toolButtonClientIcon.setIcon(
+            ray.getAppIcon(client.icon, self))
+        self.ui.labelLabel.setText(client.prettier_name())
 
         self.template_list = []
         self.ui.pushButtonAccept.setEnabled(False)
@@ -606,7 +689,9 @@ class SaveTemplateClientDialog(AbstractSaveTemplateDialog):
             self.addTemplatesToList)
 
         self.toDaemon('/ray/server/list_user_client_templates')
-
+        self.ui.lineEdit.setText(client.template_origin)
+        self.ui.lineEdit.selectAll()
+        self.ui.lineEdit.setFocus()
         self.serverStatusChanged(self._session.server_status)
 
     def serverStatusChanged(self, server_status):
@@ -615,15 +700,12 @@ class SaveTemplateClientDialog(AbstractSaveTemplateDialog):
                 ray.ServerStatus.OFF,
                 ray.ServerStatus.CLOSE) and not self.server_copying)
 
-        if server_status == ray.ServerStatus.OFF:
+        if server_status in (ray.ServerStatus.OFF, ray.ServerStatus.CLOSE):
+            self.overwrite_message_box.reject()
             self.reject()
 
         self.allowOkButton()
 
-    def addTemplatesToList(self, template_list):
-        for template in template_list:
-            self.template_list.append(template.split('/')[0])
-
 
 class ClientTrashDialog(ChildDialog):
     def __init__(self, parent, client_data):
@@ -633,15 +715,25 @@ class ClientTrashDialog(ChildDialog):
 
         self.client_data = client_data
 
+        self.ui.labelPrettierName.setText(self.client_data.prettier_name())
+        self.ui.labelDescription.setText(self.client_data.description)
         self.ui.labelExecutable.setText(self.client_data.executable_path)
         self.ui.labelId.setText(self.client_data.client_id)
-        self.ui.labelClientName.setText(self.client_data.name)
-        self.ui.labelClientIcon.setText(self.client_data.icon)
-        self.ui.labelClientLabel.setText(self.client_data.label)
-        self.ui.checkBoxSaveStop.setChecked(self.client_data.check_last_save)
         self.ui.toolButtonIcon.setIcon(QIcon.fromTheme(self.client_data.icon))
 
+        self.ui.toolButtonAdvanced.clicked.connect(self.showProperties)
         self.ui.pushButtonRemove.clicked.connect(self.removeClient)
+        self.ui.pushButtonCancel.setFocus()
+
+        self.remove_client_message_box = QMessageBox(
+            QMessageBox.Warning,
+            _translate('trashed_client', 'Remove definitely'),
+            _translate('trashed_client',
+                "Are you sure to want to remove definitely this client and all its files ?"),
+            QMessageBox.Ok | QMessageBox.Cancel,
+            self
+            )
+        self.remove_client_message_box.setDefaultButton(QMessageBox.Cancel)
 
     def serverStatusChanged(self, server_status):
         if server_status in (ray.ServerStatus.CLOSE,
@@ -649,14 +741,27 @@ class ClientTrashDialog(ChildDialog):
                              ray.ServerStatus.OUT_SAVE,
                              ray.ServerStatus.OUT_SNAPSHOT,
                              ray.ServerStatus.WAIT_USER):
+            self.remove_client_message_box.reject()
             self.reject()
 
     def removeClient(self):
+        self.remove_client_message_box.exec()
+
+        if (self.remove_client_message_box.clickedButton()
+                != self.remove_client_message_box.button(QMessageBox.Ok)):
+            return
+
         self.toDaemon(
             '/ray/trashed_client/remove_definitely',
             self.client_data.client_id)
         self.reject()
 
+    def showProperties(self):
+        properties_dialog = client_properties_dialog.ClientPropertiesDialog.create(
+            self, self.client_data)
+        properties_dialog.updateContents()
+        properties_dialog.lockWidgets()
+        properties_dialog.show()
 
 class AbortSessionDialog(ChildDialog):
     def __init__(self, parent):
@@ -722,17 +827,90 @@ class AbortClientCopyDialog(ChildDialog):
             self.reject()
 
 
+class SessionNotesDialog(ChildDialog):
+    def __init__(self, parent):
+        ChildDialog.__init__(self, parent)
+        self.ui = ui_session_notes.Ui_Dialog()
+        self.ui.setupUi(self)
+
+        if RS.settings.value('SessionNotes/geometry'):
+            self.restoreGeometry(RS.settings.value('SessionNotes/geometry'))
+        if RS.settings.value('SessionNotes/position'):
+            self.move(RS.settings.value('SessionNotes/position'))
+
+        self.message_box = None
+        self.updateSession()
+        self.ui.plainTextEdit.textChanged.connect(self.textEdited)
+
+        # use a timer to prevent osc message each time a letter is written
+        # here, a message is sent when user made not change during 400ms
+        self.timer_text = QTimer()
+        self.timer_text.setInterval(400)
+        self.timer_text.setSingleShot(True)
+        self.timer_text.timeout.connect(self.sendNotes)
+
+        self.server_off = False
+
+        self.anti_timer = False
+        self.notesUpdated()
+
+    def serverStatusChanged(self, server_status):
+        if server_status == ray.ServerStatus.OFF:
+            self.server_off = True
+            if self.message_box is not None:
+                self.message_box.close()
+            self.close()
+        else:
+            self.server_off = False
+
+    def updateSession(self):
+        self.setWindowTitle(_translate('notes_dialog', "%s Notes - %s")
+                            % (ray.APP_TITLE, self._session.name))
+        self.ui.labelSessionName.setText(self._session.name)
+
+    def textEdited(self):
+        if not self.anti_timer:
+            self.timer_text.start()
+        self.anti_timer = False
+
+    def sendNotes(self):
+        notes = self.ui.plainTextEdit.toPlainText()
+        if len(notes) >= 65000:
+            self.message_box = QMessageBox(
+                QMessageBox.Critical,
+                _translate('session_notes', 'Too long notes'),
+                _translate('session_notes',
+                           "<p>Because notes are spread to the OSC server,<br>they can't be longer than 65000 characters.<br>Sorry !</p>"),
+                QMessageBox.Cancel,
+                self)
+            self.message_box.exec()
+            self.ui.plainTextEdit.setPlainText(notes[:64999])
+            return
+
+        self._session.notes = notes
+        self.toDaemon('/ray/session/set_notes', self._session.notes)
+
+    def notesUpdated(self):
+        self.anti_timer = True
+        self.ui.plainTextEdit.setPlainText(self._session.notes)
+
+    def closeEvent(self, event):
+        RS.settings.setValue('SessionNotes/geometry', self.saveGeometry())
+        RS.settings.setValue('SessionNotes/position', self.pos())
+        if not self.server_off:
+            self.toDaemon('/ray/session/hide_notes')
+        ChildDialog.closeEvent(self, event)
+
 class OpenNsmSessionInfoDialog(ChildDialog):
     def __init__(self, parent):
         ChildDialog.__init__(self, parent)
         self.ui = ui_nsm_open_info.Ui_Dialog()
         self.ui.setupUi(self)
-
         self.ui.checkBox.stateChanged.connect(self.showThis)
-    
+
     @classmethod
     def showThis(cls, state):
-        RS.settings.setValue('OpenNsmSessionInfo', not bool(state))
+        RS.setHidden(RS.HD_OpenNsmSession, bool(state))
 
 
 class QuitAppDialog(ChildDialog):
@@ -751,6 +929,8 @@ class QuitAppDialog(ChildDialog):
             ('<strong>%s</strong>' %
              self._session.name))
 
+        if CommandLineArgs.under_nsm:
+            self.ui.pushButtonDaemon.setVisible(False)
         self.serverStatusChanged(self._session.server_status)
 
     def serverStatusChanged(self, server_status):
@@ -770,6 +950,9 @@ class QuitAppDialog(ChildDialog):
         self.toDaemon('/ray/session/abort')
 
     def leaveDaemonRunning(self):
+        if CommandLineArgs.under_nsm:
+            return
+
         self._daemon_manager.disannounce()
         QTimer.singleShot(10, QGuiApplication.quit)
 
@@ -952,6 +1135,23 @@ class StopClientNoSaveDialog(ChildDialog):
         self.client.sendPropertiesToDaemon()
 
 
+class ClientRenameDialog(ChildDialog):
+    def __init__(self, parent, client):
+        ChildDialog.__init__(self, parent)
+        self.ui = ui_client_rename.Ui_Dialog()
+        self.ui.setupUi(self)
+
+        self.client = client
+        self.ui.toolButtonIcon.setIcon(ray.getAppIcon(client.icon, self))
+        self.ui.labelClientLabel.setText(client.prettier_name())
+        self.ui.lineEdit.setText(client.prettier_name())
+        self.ui.lineEdit.selectAll()
+        self.ui.lineEdit.setFocus()
+
+    def getNewLabel(self)->str:
+        return self.ui.lineEdit.text()
+
+
 class SnapShotProgressDialog(ChildDialog):
     def __init__(self, parent):
         ChildDialog.__init__(self, parent)
@@ -1143,10 +1343,7 @@ class WaitingCloseUserDialog(ChildDialog):
         self.ui.pushButtonOk.setFocus(True)
         self.ui.pushButtonUndo.clicked.connect(self.undoClose)
         self.ui.pushButtonSkip.clicked.connect(self.skip)
-        self.ui.checkBox.setChecked(
-            bool(RS.settings.value(
-                'hide_wait_close_user_dialog', False, type=bool)))
-
+        self.ui.checkBox.setChecked(not RS.isHidden(RS.HD_WaitCloseUser))
         self.ui.checkBox.clicked.connect(self.checkBoxClicked)
 
     def serverStatusChanged(self, server_status):
@@ -1160,8 +1357,7 @@ class WaitingCloseUserDialog(ChildDialog):
         self.toDaemon('/ray/session/skip_wait_user')
 
     def checkBoxClicked(self, state):
-        RS.settings.setValue('hide_wait_close_user_dialog',
-                             bool(state), type=bool)
+        RS.setHidden(RS.HD_WaitCloseUser, bool(state))
 
 class DonationsDialog(ChildDialog):
     def __init__(self, parent, display_no_again):
@@ -1173,7 +1369,7 @@ class DonationsDialog(ChildDialog):
         self.ui.checkBox.clicked.connect(self.checkBoxClicked)
 
     def checkBoxClicked(self, state):
-        RS.settings.setValue('hide_donations', state)
+        RS.setHidden(RS.HD_Donations, state)
 
 
 class ErrorDialog(ChildDialog):
