@@ -13,6 +13,7 @@ import ui_ray_hack_copy
 import ui_client_properties
 import ui_nsm_properties
 import ui_ray_hack_properties
+import ui_ray_net_properties
 
 class RayHackCopyDialog(ChildDialog):
     def __init__(self, parent):
@@ -59,9 +60,24 @@ class ClientPropertiesDialog(ChildDialog):
             return NsmClientPropertiesDialog(window, client)
         if client.protocol == ray.Protocol.RAY_HACK:
             return RayHackClientPropertiesDialog(window, client)
+        if client.protocol == ray.Protocol.RAY_NET:
+            return RayNetClientPropertiesDialog(window, client)
 
         return ClientPropertiesDialog(window, client)
 
+    def lockWidgets(self):
+        self.ui.lineEditIcon.setReadOnly(True)
+        self.ui.lineEditLabel.setReadOnly(True)
+        self.ui.lineEditIgnoredExtensions.setReadOnly(True)
+        self.ui.checkBoxSaveStop.setEnabled(False)
+        self.ui.pushButtonSaveChanges.setVisible(False)
+
+    def setForTemplate(self, template_name):
+        self.lockWidgets()
+        self.setWindowTitle(
+            _translate('client_properties', "Properties of template %s")
+            % template_name)
+
     def setOnSecondTab(self):
         self.ui.tabWidget.setCurrentIndex(1)
 
@@ -84,6 +100,23 @@ class ClientPropertiesDialog(ChildDialog):
         icon = ray.getAppIcon(text, self)
         self.ui.toolButtonIcon.setIcon(icon)
 
+    def getCapacitiesLine(self):
+        capas_en = [c for c in self.client.capabilities.split(':') if c]
+        capas_tr = []
+        for capa in capas_en:
+            if capa == 'switch':
+                capa_tr = _translate('capabilities', 'switch')
+            elif capa == 'dirty':
+                capa_tr = _translate('capabilities', 'dirty')
+            elif capa == 'optional-gui':
+                capa_tr = _translate('capabilities', 'optional-gui')
+            else:
+                capa_tr = capa
+
+            capas_tr.append(capa_tr)
+
+        return '\n'.join(capas_tr)
+
     def saveChanges(self):
         self.client.label = self.ui.lineEditLabel.text()
         self.client.description = \
@@ -106,35 +139,27 @@ class NsmClientPropertiesDialog(ClientPropertiesDialog):
         self.nsmui_frame = QFrame()
         self.nsmui = ui_nsm_properties.Ui_Frame()
         self.nsmui.setupUi(self.nsmui_frame)
-
         self.ui.verticalLayoutProtocol.addWidget(self.nsmui_frame)
 
         self.ui.tabWidget.setTabText(1, 'NSM')
 
+    def lockWidgets(self):
+        ClientPropertiesDialog.lockWidgets(self)
+        self.nsmui.lineEditExecutable.setReadOnly(True)
+        self.nsmui.lineEditArguments.setReadOnly(True)
+        for widget in (self.nsmui.labelCapabilitiesTitle,
+                       self.nsmui.labelCapabilitiesColon,
+                       self.nsmui.labelCapabilities,
+                       self.nsmui.labelExecutableWarning):
+            widget.setVisible(False)
+
     def updateContents(self):
         ClientPropertiesDialog.updateContents(self)
         self.nsmui.labelClientName.setText(self.client.name)
+        self.nsmui.labelCapabilities.setText(self.getCapacitiesLine())
         self.nsmui.lineEditExecutable.setText(self.client.executable_path)
         self.nsmui.lineEditArguments.setText(self.client.arguments)
 
-        capas_en = [c for c in self.client.capabilities.split(':') if c]
-        capas_tr = []
-        for capa in capas_en:
-            if capa == 'switch':
-                capa_tr = _translate('capabilities', 'switch')
-            elif capa == 'dirty':
-                capa_tr = _translate('capabilities', 'dirty')
-            elif capa == 'optional-gui':
-                capa_tr = _translate('capabilities', 'optional-gui')
-            else:
-                capa_tr = capa
-
-
-            capas_tr.append(capa_tr)
-
-        capa_line = '\n'.join(capas_tr)
-        self.nsmui.labelCapabilities.setText(capa_line)
-
     def saveChanges(self):
         self.client.executable_path = self.nsmui.lineEditExecutable.text()
         self.client.arguments = self.nsmui.lineEditArguments.text()
@@ -198,6 +223,24 @@ class RayHackClientPropertiesDialog(ClientPropertiesDialog):
         self.rhack.groupBoxTestZone.toggled.connect(
             self.rhack.frameTestZone.setEnabled)
 
+    def lockWidgets(self):
+        ClientPropertiesDialog.lockWidgets(self)
+        self.rhack.lineEditExecutable.setReadOnly(True)
+        self.rhack.lineEditArguments.setReadOnly(True)
+        self.rhack.lineEditConfigFile.setReadOnly(True)
+        self.rhack.checkBoxCloseGracefully.setEnabled(False)
+        self.rhack.checkBoxTellUser.setEnabled(False)
+        self.rhack.checkBoxWaitWindow.setEnabled(False)
+        self.rhack.comboSaveSig.setEnabled(False)
+        self.rhack.comboStopSig.setEnabled(False)
+        self.rhack.groupBoxTestZone.setVisible(False)
+        self.rhack.toolButtonBrowse.setVisible(False)
+
+    def setForTemplate(self, template_name):
+        ClientPropertiesDialog.setForTemplate(self, template_name)
+        self.rhack.labelWorkingDirTitle.setVisible(False)
+        self.rhack.labelWorkingDir.setVisible(False)
+
     def updateStatus(self, status):
         self._current_status = status
         self.rhack.lineEditClientStatus.setText(clientStatusString(status))
@@ -428,3 +471,50 @@ class RayHackClientPropertiesDialog(ClientPropertiesDialog):
         ClientPropertiesDialog.showEvent(self, event)
         self._signaler.client_properties_state_changed.emit(
             self.client.client_id, True)
+
+class RayNetClientPropertiesDialog(ClientPropertiesDialog):
+    def __init__(self, parent, client):
+        ClientPropertiesDialog.__init__(self, parent, client)
+
+        self.ray_net_frame = QFrame()
+        self.rnet = ui_ray_net_properties.Ui_Frame()
+        self.rnet.setupUi(self.ray_net_frame)
+
+        self.ui.groupBoxSnapshots.setVisible(False)
+
+        self.ui.verticalLayoutProtocol.addWidget(self.ray_net_frame)
+        self.ui.tabWidget.setTabText(1, 'Ray-Net')
+
+    def lockWidgets(self):
+        ClientPropertiesDialog.lockWidgets(self)
+        self.rnet.lineEditDaemonUrl.setReadOnly(True)
+        self.rnet.lineEditSessionRoot.setReadOnly(True)
+        self.rnet.lineEditTemplate.setReadOnly(True)
+
+    def updateContents(self):
+        ClientPropertiesDialog.updateContents(self)
+        self.rnet.labelClientName.setText(self.client.name)
+        self.rnet.labelCapabilities.setText(self.getCapacitiesLine())
+        self.rnet.lineEditDaemonUrl.setText(self.client.ray_net.daemon_url)
+        self.rnet.lineEditSessionRoot.setText(self.client.ray_net.session_root)
+        self.rnet.lineEditTemplate.setText(self.client.ray_net.session_template)
+
+    def saveChanges(self):
+        new_url = self.rnet.lineEditDaemonUrl.text()
+        new_root = self.rnet.lineEditSessionRoot.text()
+        new_template = self.rnet.lineEditTemplate.text()
+
+        if ray.isValidOscUrl(new_url):
+            self.client.ray_net.daemon_url = new_url
+        if ray.isValidFullPath(new_root):
+            self.client.ray_net.session_root = new_root
+        if '/' not in new_template:
+            self.client.ray_net.session_template = new_template
+
+        self.client.sendRayNet()
+        ClientPropertiesDialog.saveChanges(self)
+
+    def changeIconwithText(self, text):
+        icon = ray.getAppIcon(text, self)
+        self.ui.toolButtonIcon.setIcon(icon)
+        self.rnet.toolButtonIcon.setIcon(icon)
