@@ -1,3 +1,5 @@
+
+import os
 import socket
 import sys
 from liblo import Address
@@ -126,6 +128,7 @@ class DaemonManager(QObject):
         self.port = src_addr.port
         self.url = src_addr.url
         self.session_root = session_root
+        CommandLineArgs.changeSessionRoot(self.session_root)
 
         self.is_nsm_locked = options & ray.Option.NSM_LOCKED
 
@@ -172,22 +175,36 @@ class DaemonManager(QObject):
             self.callDaemon()
             return
 
-        ray_control_process = QProcess()
-        ray_control_process.start("ray_control", ['get_port_gui_free'])
-        ray_control_process.waitForFinished(2000)
+        if not CommandLineArgs.force_new_daemon:
+            ray_control_process = QProcess()
+            ray_control_process.start("ray_control",
+                                    ['get_port_gui_free',
+                                    CommandLineArgs.session_root])
+            ray_control_process.waitForFinished(2000)
 
-        if ray_control_process.exitCode() == 0:
-            port_str_lines = ray_control_process.readAllStandardOutput().data().decode('utf-8')
-            port_str = port_str_lines.partition('\n')[0]
+            if ray_control_process.exitCode() == 0:
+                port_str_lines = \
+                    ray_control_process.readAllStandardOutput().data().decode('utf-8')
+                port_str = port_str_lines.partition('\n')[0]
 
-            if port_str and port_str.isdigit():
-                self.address = Address(int(port_str))
-                self.port = self.address.port
-                self.url = self.address.url
-                self.launched_before = True
-                self.is_local = True
-                self.callDaemon()
-                return
+                if port_str and port_str.isdigit():
+                    self.address = Address(int(port_str))
+                    self.port = self.address.port
+                    self.url = self.address.url
+                    self.launched_before = True
+                    self.is_local = True
+                    self.callDaemon()
+                    sys.stderr.write(
+                        "\033[92m%s\033[0m\n" % (_translate('GUI_daemon',
+                                            "Connecting GUI to existing ray-daemon port %i")
+                                    % self.port))
+
+                    if CommandLineArgs.start_session:
+                        server = GUIServerThread.instance()
+                        if server:
+                            server.send(self.address, '/ray/server/open_session',
+                                        CommandLineArgs.start_session)
+                    return
 
         server = GUIServerThread.instance()
         if not server:
@@ -196,12 +213,13 @@ class DaemonManager(QObject):
 
         # start process
         arguments = ['--gui-url', str(server.url),
+                     '--gui-pid', str(os.getpid()),
                      '--osc-port', str(self.port),
                      '--session-root', CommandLineArgs.session_root]
 
-        if CommandLineArgs.session:
+        if CommandLineArgs.start_session:
             arguments.append('--session')
-            arguments.append(CommandLineArgs.session)
+            arguments.append(CommandLineArgs.start_session)
 
         if CommandLineArgs.debug_only:
             arguments.append('--debug-only')
