@@ -49,14 +49,13 @@ class Session:
 
         # build and show Main UI
         self._main_win = MainWindow(self)
-
         self._daemon_manager.finishInit()
         server.finishInit(self)
 
         self._main_win.show()
 
         # display donations dialog under conditions
-        if not RS.settings.value('hide_donations', False, type=bool):
+        if not RS.isHidden(RS.HD_Donations):
             coreff_counter = RS.settings.value('coreff_counter', 0, type=int)
             coreff_counter += 1
             RS.settings.setValue('coreff_counter', coreff_counter)
@@ -105,35 +104,26 @@ class Session:
     def removeAllClients(self):
         self.client_list.clear()
 
-    def addFavorite(self, name, icon_name, factory, from_server=False):
+    def addFavorite(self, template_name: str, icon_name: str, factory: bool):
+        server = GUIServerThread.instance()
+        if server:
+            server.toDaemon('/ray/favorites/add', template_name,
+                            icon_name, int(factory))
+
+    def removeFavorite(self, template_name: str, factory: bool):
         for favorite in self.favorite_list:
-            if favorite.name == name and favorite.factory == factory:
-                favorite.icon = icon_name
-                return
-
-        fav = ray.Favorite(name, icon_name, factory)
-        self.favorite_list.append(fav)
-
-        self._main_win.updateFavoritesMenu()
-
-        if not from_server:
-            server = GUIServerThread.instance()
-            if server:
-                server.toDaemon('/ray/favorites/add', name,
-                                icon_name, int(factory))
-
-    def removeFavorite(self, name, factory, from_server=False):
-        for favorite in self.favorite_list:
-            if favorite.name == name and favorite.factory == factory:
-                self.favorite_list.remove(favorite)
+            if favorite.name == template_name and favorite.factory == factory:
                 break
 
-        self._main_win.updateFavoritesMenu()
+        server = GUIServerThread.instance()
+        if server:
+            server.toDaemon('/ray/favorites/remove', template_name, int(factory))
 
-        if not from_server:
-            server = GUIServerThread.instance()
-            if server:
-                server.toDaemon('/ray/favorites/remove', name, int(factory))
+    def isFavorite(self, template_name: str, factory: bool):
+        for favorite in self.favorite_list:
+            if favorite.name == template_name and favorite.factory == factory:
+                return True
+        return False
 
     def setDaemonOptions(self, options):
         self._main_win.setDaemonOptions(options)
@@ -144,7 +134,6 @@ class SignaledSession(Session):
     def __init__(self):
         Session.__init__(self)
         self._signaler.osc_receive.connect(self.oscReceive)
-
         self._daemon_manager.start()
 
     def oscReceive(self, path, args):
@@ -224,7 +213,14 @@ class SignaledSession(Session):
 
     def _ray_gui_session_notes(self, path, args):
         self.notes = args[0]
-        # TODO write the notes GUI
+        if self._main_win.notes_dialog is not None:
+            self._main_win.notes_dialog.notesUpdated()
+
+    def _ray_gui_session_notes_shown(self, path, args):
+        self._main_win.editNotes()
+
+    def _ray_gui_session_notes_hidden(self, path, args):
+        self._main_win.editNotes(close=True)
 
     def _ray_gui_session_sort_clients(self, path, args):
         new_client_list = []
@@ -236,6 +232,10 @@ class SignaledSession(Session):
 
             new_client_list.append(client)
 
+        if args == [c.client_id for c in self.client_list]:
+            # no change between existing and new order
+            return
+
         self.client_list.clear()
         self._main_win.reCreateListWidget()
 
@@ -264,6 +264,12 @@ class SignaledSession(Session):
         if client and client.protocol == ray.Protocol.RAY_HACK:
             client.updateRayHack(*args)
 
+    def _ray_gui_client_ray_net_update(self, path, args):
+        client_id = args.pop(0)
+        client = self.getClient(client_id)
+        if client and client.protocol == ray.Protocol.RAY_NET:
+            client.updateRayNet(*args)
+
     def  _ray_gui_client_switch(self, path, args):
         old_id, new_id = args
         for client in self.client_list:
@@ -301,6 +307,7 @@ class SignaledSession(Session):
     def _ray_gui_client_has_optional_gui(self, path, args):
         client_id = args[0]
         client = self.getClient(client_id)
+
         if client:
             client.setGuiEnabled()
 
@@ -324,16 +331,31 @@ class SignaledSession(Session):
             client.setNoSaveLevel(no_save_level)
 
     def _ray_gui_trash_add(self, path, args):
-        client_data = ray.ClientData.newFrom(*args)
-        trash_action = self._main_win.trashAdd(client_data)
-        trashed_client = TrashedClient(client_data, trash_action)
+        trashed_client = TrashedClient()
+        trashed_client.update(*args)
+        trash_action = self._main_win.trashAdd(trashed_client)
+        trashed_client.setMenuAction(trash_action)
         self.trashed_clients.append(trashed_client)
 
+    def _ray_gui_trash_ray_hack_update(self, path, args):
+        client_id = args.pop(0)
+        for trashed_client in self.trashed_clients:
+            if trashed_client.client_id == client_id:
+                trashed_client.ray_hack = ray.RayHack.newFrom(*args)
+                break
+
+    def _ray_gui_trash_ray_net_update(self, path, args):
+        client_id = args.pop(0)
+        for trashed_client in self.trashed_clients:
+            if trashed_client.client_id == client_id:
+                trashed_client.ray_net = ray.RayNet.newFrom(*args)
+                break
+
     def _ray_gui_trash_remove(self, path, args):
         client_id = args[0]
 
         for trashed_client in self.trashed_clients:
-            if trashed_client.data.client_id == client_id:
+            if trashed_client.client_id == client_id:
                 break
         else:
             return
@@ -346,12 +368,34 @@ class SignaledSession(Session):
         self._main_win.trashClear()
 
     def _ray_gui_favorites_added(self, path, args):
-        name, icon_name, int_factory = args
-        self.addFavorite(name, icon_name, bool(int_factory), True)
+        template_name, icon_name, int_factory = args
 
-    def _ray_gui_favorites_remove(self, path, args):
-        name, int_factory = args
-        self.removeFavorite(name, bool(int_factory), True)
+        for favorite in self.favorite_list:
+            if (favorite.name == template_name
+                    and favorite.factory == bool(int_factory)):
+                # favorite already exists, update the icon
+                favorite.icon = icon_name
+                break
+        else:
+            fav = ray.Favorite(template_name, icon_name, bool(int_factory))
+            self.favorite_list.append(fav)
+            self._signaler.favorite_added.emit(
+                template_name, icon_name, bool(int_factory))
+        self._main_win.updateFavoritesMenu()
+
+    def _ray_gui_favorites_removed(self, path, args):
+        template_name, int_factory = args
+
+        for favorite in self.favorite_list:
+            if (favorite.name == template_name
+                    and favorite.factory == bool(int_factory)):
+                break
+        else:
+            return
+
+        self.favorite_list.remove(favorite)
+        self._signaler.favorite_removed.emit(template_name, bool(int_factory))
+        self._main_win.updateFavoritesMenu()
 
     def _ray_gui_script_info(self, path, args):
         text = args[0]
