@@ -23,6 +23,8 @@ class Signaler(QObject):
     daemon_url_changed = pyqtSignal(str)
 
     client_template_update = pyqtSignal(list)
+    client_template_ray_hack_update = pyqtSignal(list)
+    client_template_ray_net_update = pyqtSignal(list)
 
     root_changed = pyqtSignal(str)
 
@@ -30,6 +32,8 @@ class Signaler(QObject):
 
     client_properties_state_changed = pyqtSignal(str, bool)
 
+    favorite_added = pyqtSignal(str, str, bool)
+    favorite_removed = pyqtSignal(str, bool)
 
     def __init__(self):
         QObject.__init__(self)
