@@ -4,7 +4,7 @@ from PyQt5.QtCore import pyqtSlot, QSize
 
 import ray
 from gui_server_thread import GUIServerThread
-from gui_tools import clientStatusString, _translate, isDarkTheme
+from gui_tools import clientStatusString, _translate, isDarkTheme, RayIcon
 import child_dialogs
 import snapshots_dialog
 
@@ -18,7 +18,7 @@ class ClientSlot(QFrame):
         server = GUIServerThread.instance()
         if server:
             server.toDaemon(*args)
-    
+
     def __init__(self, list_widget, client):
         QFrame.__init__(self)
         self.ui = ui_client_slot.Ui_ClientSlotWidget()
@@ -29,41 +29,37 @@ class ClientSlot(QFrame):
         self.client = client
         self._main_win = self.client._session._main_win
 
-        self.is_dirty_able = False
-        self.gui_visible = False
+        self.gui_state = False
 
         self.ui.toolButtonGUI.setVisible(False)
-
-        if self.client.protocol == ray.Protocol.RAY_HACK:
-            self.showGuiButton()
+        if client.protocol != ray.Protocol.RAY_HACK:
+            self.ui.toolButtonHack.setVisible(False)
 
         # connect buttons to functions
-        self.ui.toolButtonGUI.toggleGui.connect(self.toggleGui)
+        self.ui.toolButtonHack.orderHackVisibility.connect(self.orderHackVisibility)
+        self.ui.toolButtonGUI.clicked.connect(self.changeGuiState)
         self.ui.startButton.clicked.connect(self.startClient)
         self.ui.stopButton.clicked.connect(self.stopClient)
         self.ui.killButton.clicked.connect(self.killClient)
         self.ui.saveButton.clicked.connect(self.saveClient)
         self.ui.closeButton.clicked.connect(self.trashClient)
         self.ui.lineEditClientStatus.statusPressed.connect(self.abortCopy)
-        # self.ui.ClientName.name_changed.connect(self.updateLabel)
-
-        # prevent "stopped" status displayed at client switch
-        #self.ui.lineEditClientStatus,.setTextNow(
-                          #clientStatusString(self.client.status))
 
         self.icon_on = QIcon()
         self.icon_off = QIcon()
 
         self.ui.actionSaveAsApplicationTemplate.triggered.connect(
             self.saveAsApplicationTemplate)
-        self.ui.actionProperties.triggered.connect(
-            self.client.showPropertiesDialog)
+        self.ui.actionRename.triggered.connect(self.renameDialog)
         self.ui.actionReturnToAPreviousState.triggered.connect(
             self.openSnapshotsDialog)
+        self.ui.actionProperties.triggered.connect(
+            self.client.showPropertiesDialog)
 
         self.menu = QMenu(self)
 
         self.menu.addAction(self.ui.actionSaveAsApplicationTemplate)
+        self.menu.addAction(self.ui.actionRename)
         self.menu.addAction(self.ui.actionReturnToAPreviousState)
         self.menu.addAction(self.ui.actionProperties)
 
@@ -72,99 +68,19 @@ class ClientSlot(QFrame):
 
         self.ui.iconButton.setMenu(self.menu)
 
-        self.saveIcon = QIcon()
-        self.saveIcon.addPixmap(
-            QPixmap(':scalable/breeze/document-save'),
-            QIcon.Normal,
-            QIcon.Off)
-        self.saveIcon.addPixmap(
-            QPixmap(':scalable/breeze/disabled/document-save'),
-            QIcon.Disabled,
-            QIcon.Off)
+        dark = isDarkTheme(self)
+
+        self.saveIcon = RayIcon('document-save', dark)
+        self.savedIcon = RayIcon('document-saved', dark)
+        self.unsavedIcon = RayIcon('document-unsaved', dark)
+        self.noSaveIcon = RayIcon('document-nosave', dark)
+        self.icon_visible = RayIcon('visibility', dark)
+        self.icon_invisible = RayIcon('hint', dark)
+        self.ui.startButton.setIcon(RayIcon('media-playback-start', dark))
+        self.ui.stopButton.setIcon(RayIcon('media-playback-stop', dark))
+        self.ui.closeButton.setIcon(RayIcon('window-close', dark))
         self.ui.saveButton.setIcon(self.saveIcon)
 
-        self.savedIcon = QIcon()
-        self.savedIcon.addPixmap(
-            QPixmap(':scalable/breeze/document-saved'),
-            QIcon.Normal,
-            QIcon.Off)
-
-        self.unsavedIcon = QIcon()
-        self.unsavedIcon.addPixmap(
-            QPixmap(':scalable/breeze/document-unsaved'),
-            QIcon.Normal,
-            QIcon.Off)
-
-        self.noSaveIcon = QIcon()
-        self.noSaveIcon.addPixmap(
-            QPixmap(':scalable/breeze/document-nosave'),
-            QIcon.Normal,
-            QIcon.Off)
-
-        # choose button colors
-        if isDarkTheme(self):
-            startIcon = QIcon()
-            startIcon.addPixmap(
-                QPixmap(':scalable/breeze-dark/media-playback-start'),
-                QIcon.Normal,
-                QIcon.Off)
-            startIcon.addPixmap(
-                QPixmap(':scalable/breeze-dark/disabled/media-playback-start'),
-                QIcon.Disabled,
-                QIcon.Off)
-            self.ui.startButton.setIcon(startIcon)
-
-            stopIcon = QIcon()
-            stopIcon.addPixmap(
-                QPixmap(':scalable/breeze-dark/media-playback-stop'),
-                QIcon.Normal,
-                QIcon.Off)
-            stopIcon.addPixmap(
-                QPixmap(':scalable/breeze-dark/disabled/media-playback-stop'),
-                QIcon.Disabled,
-                QIcon.Off)
-            self.ui.stopButton.setIcon(stopIcon)
-
-            self.saveIcon = QIcon()
-            self.saveIcon.addPixmap(
-                QPixmap(':scalable/breeze-dark/document-save'),
-                QIcon.Normal,
-                QIcon.Off)
-            self.saveIcon.addPixmap(
-                QPixmap(':scalable/breeze-dark/disabled/document-save'),
-                QIcon.Disabled,
-                QIcon.Off)
-            self.ui.saveButton.setIcon(self.saveIcon)
-
-            self.savedIcon = QIcon()
-            self.savedIcon.addPixmap(
-                QPixmap(':scalable/breeze-dark/document-saved'),
-                QIcon.Normal,
-                QIcon.Off)
-
-            self.unsavedIcon = QIcon()
-            self.unsavedIcon.addPixmap(
-                QPixmap(':scalable/breeze-dark/document-unsaved'),
-                QIcon.Normal,
-                QIcon.Off)
-
-            self.noSaveIcon = QIcon()
-            self.noSaveIcon.addPixmap(
-                QPixmap(':scalable/breeze-dark/document-nosave'),
-                QIcon.Normal,
-                QIcon.Off)
-
-            closeIcon = QIcon()
-            closeIcon.addPixmap(
-                QPixmap(':scalable/breeze-dark/window-close'),
-                QIcon.Normal,
-                QIcon.Off)
-            closeIcon.addPixmap(
-                QPixmap(':scalable/breeze-dark/disabled/window-close'),
-                QIcon.Disabled,
-                QIcon.Off)
-            self.ui.closeButton.setIcon(closeIcon)
-
         self.ubuntu_font = QFont(
             QFontDatabase.applicationFontFamilies(0)[0], 8)
         self.ubuntu_font_cond = QFont(
@@ -174,12 +90,18 @@ class ClientSlot(QFrame):
 
         self.ui.killButton.setVisible(False)
 
+        if ':optional-gui:' in self.client.capabilities:
+            self.setGuiState(self.client.gui_state)
+            self.ui.toolButtonGUI.setVisible(True)
+
+        if self.client.has_dirty:
+            self.setDirtyState(self.client.dirty_state)
+
         self.updateClientData()
 
     def clientId(self):
         return self.client.client_id
 
-
     def startClient(self):
         self.toDaemon('/ray/client/resume', self.clientId())
 
@@ -201,7 +123,8 @@ class ClientSlot(QFrame):
         self._main_win.abortCopyClient(self.clientId())
 
     def saveAsApplicationTemplate(self):
-        dialog = child_dialogs.SaveTemplateClientDialog(self._main_win)
+        dialog = child_dialogs.SaveTemplateClientDialog(
+            self._main_win, self.client)
         dialog.exec()
         if not dialog.result():
             return
@@ -219,13 +142,20 @@ class ClientSlot(QFrame):
             self.toDaemon('/ray/client/open_snapshot',
                           self.clientId(), snapshot)
 
+    def renameDialog(self):
+        dialog = child_dialogs.ClientRenameDialog(self._main_win,
+                                                  self.client)
+        dialog.exec()
+        if dialog.result():
+            self.client.label = dialog.getNewLabel()
+            self.client.sendPropertiesToDaemon()
+
     def updateLabel(self, label):
         self._main_win.updateClientLabel(self.clientId(), label)
 
     def updateClientData(self):
         # set main label
-        label = self.client.label if self.client.label else self.client.name
-        self.ui.ClientName.setText(label)
+        self.ui.ClientName.setText(self.client.prettier_name())
 
         # set tool tip
         tool_tip = "<html><head/><body>"
@@ -256,6 +186,34 @@ class ClientSlot(QFrame):
                     ray.ClientStatus.STOPPED,
                     ray.ClientStatus.PRECOPY)))
 
+        self.ui.toolButtonGUI.setVisible(
+            bool(':optional-gui:' in self.client.capabilities))
+
+        if self.client.executable_path in ('ray-proxy', 'nsm-proxy'):
+            if isDarkTheme(self):
+                self.icon_visible = QIcon()
+                self.icon_visible.addPixmap(
+                    QPixmap(':scalable/breeze-dark/emblem-symbolic-link'),
+                    QIcon.Normal, QIcon.Off)
+                self.icon_invisible = QIcon()
+                self.icon_invisible.addPixmap(
+                    QPixmap(':scalable/breeze-dark/link'),
+                    QIcon.Normal, QIcon.Off)
+                self.icon_invisible.addPixmap(
+                    QPixmap(':scalable/breeze-dark/disabled/link'),
+                    QIcon.Disabled, QIcon.Off)
+            else:
+                self.icon_visible = QIcon()
+                self.icon_visible.addPixmap(
+                    QPixmap(':scalable/breeze/emblem-symbolic-link'),
+                    QIcon.Normal, QIcon.Off)
+                self.icon_invisible = QIcon()
+                self.icon_invisible.addPixmap(
+                    QPixmap(':scalable/breeze/link'), QIcon.Normal, QIcon.Off)
+                self.icon_invisible.addPixmap(
+                    QPixmap(':scalable/breeze/disabled/link'),
+                    QIcon.Disabled, QIcon.Off)
+
     def grayIcon(self, gray):
         if gray:
             self.ui.iconButton.setIcon(self.icon_off)
@@ -298,7 +256,7 @@ class ClientSlot(QFrame):
             self.ui.closeButton.setEnabled(True)
             self.ui.ClientName.setStyleSheet('QLabel {font-weight : normal}')
             self.ui.ClientName.setEnabled(False)
-            self.ui.toolButtonGUI.setEnabled(ray_hack)
+            self.ui.toolButtonGUI.setEnabled(False)
             self.grayIcon(True)
 
             self.ui.stopButton.setVisible(True)
@@ -306,6 +264,9 @@ class ClientSlot(QFrame):
 
             self.ui.saveButton.setIcon(self.saveIcon)
 
+            if not ray_hack:
+                self.setGuiState(False)
+
         elif status == ray.ClientStatus.PRECOPY:
             self.ui.startButton.setEnabled(False)
             self.ui.stopButton.setEnabled(False)
@@ -313,7 +274,7 @@ class ClientSlot(QFrame):
             self.ui.closeButton.setEnabled(True)
             self.ui.ClientName.setStyleSheet('QLabel {font-weight : normal}')
             self.ui.ClientName.setEnabled(False)
-            self.ui.toolButtonGUI.setEnabled(ray_hack)
+            self.ui.toolButtonGUI.setEnabled(False)
             self.grayIcon(True)
 
             self.ui.stopButton.setVisible(True)
@@ -335,42 +296,44 @@ class ClientSlot(QFrame):
         else:
             self.ui.lineEditClientStatus.setText('')
 
+    def setHackButtonState(self, state: bool):
+        self.ui.toolButtonHack.setChecked(state)
+
     def showGuiButton(self):
+        self.ui.toolButtonGUI.setIcon(self.icon_invisible)
         self.ui.toolButtonGUI.setVisible(True)
-        if self.client.protocol == ray.Protocol.RAY_HACK:
-            self.ui.toolButtonGUI.setText(
-                _translate('client_slot', 'Hack'))
-            self.ui.toolButtonGUI.setToolTip(
-                _translate('client_slot',
-                           'Display client Ray-Hack properties'))
-
-        elif self.client.executable_path in ('nsm-proxy', 'ray-proxy'):
-            self.ui.toolButtonGUI.setText(_translate('client_slot', 'proxy'))
-            self.ui.toolButtonGUI.setToolTip(
-                _translate('client_slot', 'Display proxy window'))
-
-    def setGuiState(self, state):
-        self.gui_visible = state
-        self.ui.toolButtonGUI.setChecked(state)
-
-    def toggleGui(self):
-        if self.client.protocol == ray.Protocol.RAY_HACK:
-            if self.gui_visible:
-                self.client.properties_dialog.hide()
-            else:
-                self.client.showPropertiesDialog(second_tab=True)
 
+    def setGuiState(self, state: bool):
+        if state:
+            self.ui.toolButtonGUI.setIcon(self.icon_visible)
         else:
-            if not self.gui_visible:
-                self.toDaemon('/ray/client/show_optional_gui',
-                              self.clientId())
-            else:
-                self.toDaemon('/ray/client/hide_optional_gui',
-                              self.clientId())
+            self.ui.toolButtonGUI.setIcon(self.icon_invisible)
+
+        self.gui_state = state
+
+    def changeGuiState(self):
+        if self.gui_state:
+            self.toDaemon('/ray/client/hide_optional_gui', self.clientId())
+        else:
+            self.toDaemon('/ray/client/show_optional_gui', self.clientId())
+
+    def orderHackVisibility(self, state):
+        if self.client.protocol != ray.Protocol.RAY_HACK:
+            return
+
+        if state:
+            self.client.showPropertiesDialog(second_tab=True)
+        else:
+            self.client.properties_dialog.hide()
+
+        #if state:
+            #self.toDaemon('/ray/client/show_optional_gui',
+                            #self.clientId())
+        #else:
+            #self.toDaemon('/ray/client/hide_optional_gui',
+                            #self.clientId())
 
     def setDirtyState(self, bool_dirty):
-        self.is_dirty_able = True
-
         if bool_dirty:
             self.ui.saveButton.setIcon(self.unsavedIcon)
         else:
@@ -422,7 +385,7 @@ class ListWidgetClients(QListWidget):
         server = GUIServerThread.instance()
         if server:
             server.toDaemon(*args)
-    
+
     def __init__(self, parent):
         QListWidget.__init__(self, parent)
         self.last_n = 0
@@ -471,7 +434,7 @@ class ListWidgetClients(QListWidget):
             item = self.item(i)
             if item.getClientId() == client_id:
                 widget = item.f_widget
-                widget.setGuiState(bool_visible)
+                widget.setHackButtonState(bool_visible)
                 break
 
     def setSession(self, session):
@@ -509,7 +472,7 @@ class ListWidgetClients(QListWidget):
         if not self.itemAt(event.pos()):
             self.setCurrentRow(-1)
 
-            if (self._session
+            if (self._session is not None
                     and not self._session.server_status in (
                         ray.ServerStatus.OFF,
                         ray.ServerStatus.CLOSE,
