@@ -1,11 +1,12 @@
 import time
+import os
 from PyQt5.QtWidgets import (QApplication, QMainWindow, QMenu, QDialog,
                              QMessageBox, QToolButton, QAbstractItemView)
-from PyQt5.QtGui import QIcon
-from PyQt5.QtCore import QTimer, pyqtSlot
+from PyQt5.QtGui import QIcon, QDesktopServices
+from PyQt5.QtCore import QTimer, pyqtSlot, QUrl, QLocale
 
 from gui_tools import (RS, RayIcon, CommandLineArgs, _translate,
-                       serverStatusString, isDarkTheme)
+                       serverStatusString, isDarkTheme, getCodeRoot)
 import add_application_dialog
 import child_dialogs
 import snapshots_dialog
@@ -13,17 +14,17 @@ from gui_server_thread import GUIServerThread
 
 import ray
 import list_widget_clients
+import nsm_child
 
 import ui_raysession
 
-
 class MainWindow(QMainWindow):
     @classmethod
     def toDaemon(cls, *args):
         server = GUIServerThread.instance()
         if server:
             server.toDaemon(*args)
-    
+
     def __init__(self, session):
         QMainWindow.__init__(self)
         self.ui = ui_raysession.Ui_MainWindow()
@@ -36,6 +37,8 @@ class MainWindow(QMainWindow):
         self.mouse_is_inside = False
         self.terminate_request = False
 
+        self.notes_dialog = None
+
         # timer for keep focus while client opening
         self.timer_raisewin = QTimer()
         self.timer_raisewin.setInterval(50)
@@ -89,10 +92,8 @@ class MainWindow(QMainWindow):
             self.ui.actionSaveSession)
         self.ui.toolButtonAbortSession.setDefaultAction(
             self.ui.actionAbortSession)
-        self.ui.toolButtonDuplicateSession.setDefaultAction(
-            self.ui.actionDuplicateSession)
-        self.ui.toolButtonSaveTemplateSession.setDefaultAction(
-            self.ui.actionSaveTemplateSession)
+        self.ui.toolButtonNotes.setDefaultAction(
+            self.ui.actionSessionNotes)
         self.ui.toolButtonFileManager.setDefaultAction(
             self.ui.actionOpenSessionFolder)
         self.ui.toolButtonAddApplication.setDefaultAction(
@@ -114,10 +115,18 @@ class MainWindow(QMainWindow):
         self.ui.actionAbortSession.triggered.connect(self.abortSession)
         self.ui.actionRenameSession.triggered.connect(
             self.renameSessionAction)
+        self.ui.actionRenameSession_2.triggered.connect(
+            self.renameSessionAction)
         self.ui.actionDuplicateSession.triggered.connect(
             self.duplicateSession)
+        self.ui.actionDuplicateSession_2.triggered.connect(
+            self.duplicateSession)
         self.ui.actionSaveTemplateSession.triggered.connect(
             self.saveTemplateSession)
+        self.ui.actionSaveTemplateSession_2.triggered.connect(
+            self.saveTemplateSession)
+        self.ui.actionSessionNotes.triggered.connect(
+            self.toggleNotesVisibility)
         self.ui.actionReturnToAPreviousState.triggered.connect(
             self.returnToAPreviousState)
         self.ui.actionOpenSessionFolder.triggered.connect(
@@ -133,15 +142,29 @@ class MainWindow(QMainWindow):
             self.autoSnapshotToggled)
         self.ui.actionSessionScripts.triggered.connect(
             self.sessionScriptsToggled)
+        self.ui.actionRememberOptionalGuiStates.triggered.connect(
+            self.rememberOptionalGuiStatesToggled)
         self.ui.actionAboutRaySession.triggered.connect(self.aboutRaySession)
         self.ui.actionAboutQt.triggered.connect(QApplication.aboutQt)
+        self.ui.actionOnlineManual.triggered.connect(self.onlineManual)
+        self.ui.actionInternalManual.triggered.connect(self.internalManual)
         self.ui.actionDonate.triggered.connect(self.donate)
+        self.ui.actionMakeReappearDialogs.triggered.connect(
+            self.makeAllDialogsReappear)
 
         self.ui.lineEditServerStatus.statusPressed.connect(
             self.statusBarPressed)
         self.ui.stackedWidgetSessionName.name_changed.connect(
             self.renameSessionConditionnaly)
 
+        # set session menu
+        self.session_menu = QMenu()
+        self.session_menu.addAction(self.ui.actionSaveTemplateSession_2)
+        self.session_menu.addAction(self.ui.actionDuplicateSession_2)
+        self.session_menu.addAction(self.ui.actionRenameSession_2)
+        self.ui.toolButtonSessionMenu.setPopupMode(QToolButton.InstantPopup)
+        self.ui.toolButtonSessionMenu.setMenu(self.session_menu)
+
         # set control menu
         self.controlMenu = QMenu()
         self.controlMenu.addAction(self.ui.actionShowMenuBar)
@@ -153,6 +176,9 @@ class MainWindow(QMainWindow):
         self.controlMenu.addAction(self.ui.actionAutoSnapshot)
         self.controlMenu.addAction(self.ui.actionDesktopsMemory)
         self.controlMenu.addAction(self.ui.actionSessionScripts)
+        self.controlMenu.addAction(self.ui.actionRememberOptionalGuiStates)
+        self.controlMenu.addSeparator()
+        self.controlMenu.addAction(self.ui.actionMakeReappearDialogs)
 
         self.controlToolButton = self.ui.toolBar.widgetForAction(
             self.ui.actionControlMenu)
@@ -212,18 +238,30 @@ class MainWindow(QMainWindow):
         self.ui.actionReturnToAPreviousState.setIcon(
                                         RayIcon('media-seek-backward', dark))
 
+        self.ui.actionRememberOptionalGuiStates.setIcon(
+            RayIcon('visibility', dark))
         self.ui.trashButton.setIcon(RayIcon('trash-empty', dark))
 
         self.ui.actionDuplicateSession.setIcon(
             RayIcon('xml-node-duplicate', dark))
+        self.ui.actionDuplicateSession_2.setIcon(
+            RayIcon('xml-node-duplicate', dark))
         self.ui.actionSaveTemplateSession.setIcon(
             RayIcon('document-save-as-template', dark))
+        self.ui.actionSaveTemplateSession_2.setIcon(
+            RayIcon('document-save-as-template', dark))
         self.ui.actionCloseSession.setIcon(RayIcon('window-close', dark))
         self.ui.actionAbortSession.setIcon(RayIcon('list-remove', dark))
         self.ui.actionSaveSession.setIcon(RayIcon('document-save', dark))
         self.ui.toolButtonSaveSession.setIcon(RayIcon('document-save', dark))
+        self.ui.actionSessionNotes.setIcon(RayIcon('notes', dark))
+        self.ui.toolButtonNotes.setIcon(RayIcon('notes', dark))
         self.ui.actionDesktopsMemory.setIcon(RayIcon('view-list-icons', dark))
 
+        self.ui.toolButtonSessionMenu.setIcon(RayIcon('application-menu', dark))
+
+        self.ui.listWidget.setSession(self._session)
+
         self.setNsmLocked(CommandLineArgs.under_nsm)
 
         self.script_info_dialog = None
@@ -274,7 +312,7 @@ class MainWindow(QMainWindow):
         self.ui.actionAbortSession.setEnabled(not nsm_locked)
 
         self.ui.toolBar.setVisible(not nsm_locked)
-        self.ui.toolButtonDuplicateSession.setVisible(not nsm_locked)
+        self.ui.toolButtonNoRole.setVisible(nsm_locked)
         self.ui.toolButtonAbortSession.setVisible(not nsm_locked)
         self.ui.closeButton.setVisible(not nsm_locked)
         self.ui.toolButtonControl2.setVisible(nsm_locked)
@@ -283,6 +321,8 @@ class MainWindow(QMainWindow):
             nsm_locked and not CommandLineArgs.out_daemon)
         self.ui.actionRenameSession.setEnabled(
             nsm_locked and not CommandLineArgs.out_daemon)
+        self.ui.actionRenameSession_2.setEnabled(
+            nsm_locked and not CommandLineArgs.out_daemon)
 
     def setDaemonOptions(self, options):
         self.ui.actionBookmarkSessionFolder.setChecked(
@@ -293,6 +333,8 @@ class MainWindow(QMainWindow):
             bool(options & ray.Option.SNAPSHOTS))
         self.ui.actionSessionScripts.setChecked(
             bool(options & ray.Option.SESSION_SCRIPTS))
+        self.ui.actionRememberOptionalGuiStates.setChecked(
+            bool(options & ray.Option.GUI_STATES))
 
         has_wmctrl = bool(options & ray.Option.HAS_WMCTRL)
         self.ui.actionDesktopsMemory.setEnabled(has_wmctrl)
@@ -328,16 +370,24 @@ class MainWindow(QMainWindow):
             self.timer_raisewin.stop()
 
     def bookmarkSessionFolderToggled(self, state):
-        self.toDaemon('/ray/option/bookmark_session_folder', int(state))
+        self.setOption(ray.Option.BOOKMARK_SESSION, state)
 
     def desktopsMemoryToggled(self, state):
-        self.toDaemon('/ray/option/desktops_memory', int(state))
+        self.setOption(ray.Option.DESKTOPS_MEMORY, state)
 
     def autoSnapshotToggled(self, state):
-        self.toDaemon('/ray/option/snapshots', int(state))
+        self.setOption(ray.Option.SNAPSHOTS, state)
 
     def sessionScriptsToggled(self, state):
-        self.toDaemon('/ray/option/session_scripts', int(state))
+        self.setOption(ray.Option.SESSION_SCRIPTS, state)
+
+    def rememberOptionalGuiStatesToggled(self, state):
+        self.setOption(ray.Option.GUI_STATES, state)
+
+    def setOption(self, option: int, state: bool):
+        if not state:
+            option = -option
+        self.toDaemon('/ray/server/set_option', option)
 
     def flashOpen(self):
         for client in self._session.client_list:
@@ -396,8 +446,7 @@ class MainWindow(QMainWindow):
 
         if template_name.startswith('///'):
             if template_name == '///' + ray.factory_session_templates[1]:
-                if not RS.settings.value('hide_jack_config_script_dialog',
-                                        False, type=bool):
+                if not RS.isHidden(RS.HD_JackConfigScript):
                     # display jack_config_script info dialog
                     # and manage ray-jack_checker auto_start
 
@@ -410,8 +459,7 @@ class MainWindow(QMainWindow):
                     if not dialog.result():
                         return
 
-                    RS.settings.setValue('hide_jack_config_script_dialog',
-                                         dialog.notAgainValue())
+                    RS.setHidden(RS.HD_JackConfigScript, dialog.notAgainValue())
 
                     autostart_jack_checker = dialog.autostartValue()
                     action = 'set_jack_checker_autostart'
@@ -421,10 +469,8 @@ class MainWindow(QMainWindow):
                     self.toDaemon('/ray/server/exotic_action', action)
 
             elif template_name == '///' + ray.factory_session_templates[2]:
-                if not RS.settings.value('hide_session_scripts_dialog',
-                                        False, type=bool):
+                if not RS.isHidden(RS.HD_SessionScripts):
                     # display session scripts info dialog
-
                     session_path = "%s/%s" % (CommandLineArgs.session_root,
                                               session_short_path)
 
@@ -434,8 +480,7 @@ class MainWindow(QMainWindow):
                     if not dialog.result():
                         return
 
-                    RS.settings.setValue('hide_session_scripts_dialog',
-                                         dialog.notAgainValue())
+                    RS.setHidden(RS.HD_SessionScripts, dialog.notAgainValue())
 
         self.toDaemon('/ray/server/new_session', session_short_path,
                       template_name)
@@ -518,9 +563,45 @@ class MainWindow(QMainWindow):
         dialog = child_dialogs.DonationsDialog(self, display_no_again)
         dialog.exec()
 
+    def onlineManual(self):
+        QDesktopServices.openUrl(QUrl('http://raysession.tuxfamily.org/manual'))
+
+    def internalManual(self):
+        short_locale = 'en'
+        manual_dir = "%s/manual" % getCodeRoot()
+        locale_str = QLocale.system().name()
+        if (len(locale_str) > 2 and '_' in locale_str
+                and os.path.isfile(
+                    "%s/%s/manual.html" % (manual_dir, locale_str[:2]))):
+            short_locale = locale_str[:2]
+
+        QDesktopServices.openUrl(
+            QUrl("%s/%s/manual.html" % (manual_dir, short_locale)))
+
     def saveSession(self):
         self.toDaemon('/ray/session/save')
 
+    def toggleNotesVisibility(self):
+        if (self.notes_dialog is None or not self.notes_dialog.isVisible()):
+            self.toDaemon('/ray/session/show_notes')
+        else:
+            self.toDaemon('/ray/session/hide_notes')
+
+    def editNotes(self, close=False):
+        icon_str = 'notes'
+        if close:
+            if self._session.notes:
+                icon_str = 'notes-nonempty'
+            if self.notes_dialog is not None and self.notes_dialog.isVisible():
+                self.notes_dialog.close()
+        else:
+            if self.notes_dialog is None:
+                self.notes_dialog = child_dialogs.SessionNotesDialog(self)
+            self.notes_dialog.show()
+            icon_str = 'notes-editing'
+
+        self.ui.actionSessionNotes.setIcon(RayIcon(icon_str, isDarkTheme(self)))
+
     def addApplication(self):
         if self._session.server_status in (
                 ray.ServerStatus.CLOSE,
@@ -638,12 +719,6 @@ class MainWindow(QMainWindow):
     def renameSessionConditionnaly(self, new_session_name):
         self.toDaemon('/ray/session/rename', new_session_name)
 
-    def addFavorite(self, name, icon_name, factory):
-        self._session.addFavorite(name, icon_name, factory, True)
-
-    def removeFavorite(self, name, factory):
-        self._session.removeFavorite(name, factory, True)
-
     def showSnapshotProgressDialog(self):
         if self.progress_dialog_visible:
             return
@@ -719,10 +794,14 @@ class MainWindow(QMainWindow):
         if session_name:
             self.setWindowTitle('%s - %s' % (ray.APP_TITLE, session_name))
             self.ui.stackedWidgetSessionName.setText(session_name)
+            if self.notes_dialog is not None:
+                self.notes_dialog.updateSession()
         else:
             self.setWindowTitle(ray.APP_TITLE)
             self.ui.stackedWidgetSessionName.setText(
                 _translate('main view', 'No Session Loaded'))
+            if self.notes_dialog is not None:
+                self.notes_dialog.hide()
 
     def setSessionNameEditable(self, bool_set_edit):
         self.ui.stackedWidgetSessionName.setEditable(bool_set_edit)
@@ -732,7 +811,7 @@ class MainWindow(QMainWindow):
         error_dialog.exec()
 
     def openingNsmSession(self):
-        if not RS.settings.value('OpenNsmSessionInfo', True, type=bool):
+        if RS.isHidden(RS.HD_OpenNsmSession):
             return
 
         dialog = child_dialogs.OpenNsmSessionInfoDialog(self)
@@ -772,11 +851,14 @@ class MainWindow(QMainWindow):
             self.ui.actionCloseSession.setEnabled(False)
             self.ui.actionAbortSession.setEnabled(True)
             self.ui.actionDuplicateSession.setEnabled(False)
+            self.ui.actionDuplicateSession_2.setEnabled(False)
             self.ui.actionSaveTemplateSession.setEnabled(False)
+            self.ui.actionSaveTemplateSession_2.setEnabled(False)
             self.ui.actionReturnToAPreviousState.setEnabled(False)
             self.ui.actionAddApplication.setEnabled(False)
             self.ui.actionAddExecutable.setEnabled(False)
             self.ui.actionOpenSessionFolder.setEnabled(True)
+            self.ui.actionSessionNotes.setEnabled(False)
             return
 
         close_or_off = bool(
@@ -794,9 +876,12 @@ class MainWindow(QMainWindow):
             not bool(server_status in (ray.ServerStatus.CLOSE,
                                        ray.ServerStatus.OFF)))
         self.ui.actionDuplicateSession.setEnabled(not close_or_off)
+        self.ui.actionDuplicateSession_2.setEnabled(not close_or_off)
         self.ui.actionReturnToAPreviousState.setEnabled(not close_or_off)
         self.ui.actionRenameSession.setEnabled(ready)
+        self.ui.actionRenameSession_2.setEnabled(ready)
         self.ui.actionSaveTemplateSession.setEnabled(not close_or_off)
+        self.ui.actionSaveTemplateSession_2.setEnabled(not close_or_off)
         self.ui.actionAddApplication.setEnabled(not close_or_off)
         self.ui.actionAddExecutable.setEnabled(not close_or_off)
         self.ui.toolButtonFavorites.setEnabled(
@@ -805,6 +890,8 @@ class MainWindow(QMainWindow):
             bool(self._session.favorite_list and not close_or_off))
         self.ui.actionOpenSessionFolder.setEnabled(
             bool(server_status != ray.ServerStatus.OFF))
+        self.ui.actionSessionNotes.setEnabled(
+            bool(server_status != ray.ServerStatus.OFF))
 
         self.ui.stackedWidgetSessionName.setEditable(
             ready and self._session.is_renameable)
@@ -834,20 +921,15 @@ class MainWindow(QMainWindow):
                 self._daemon_manager.stop()
 
         if server_status == ray.ServerStatus.WAIT_USER:
-            if not RS.settings.value(
-                    'hide_wait_close_user_dialog', False, type=bool):
+            if not RS.isHidden(RS.HD_WaitCloseUser):
                 dialog = child_dialogs.WaitingCloseUserDialog(self)
                 dialog.exec()
 
-
-    def trashAdd(self, client_data):
-        prettier_name = client_data.name
-        if client_data.label:
-            prettier_name = client_data.label
-
+    def trashAdd(self, trashed_client):
         act_x_trashed = self.trashMenu.addAction(
-            QIcon.fromTheme(client_data.icon), prettier_name)
-        act_x_trashed.setData(client_data.client_id)
+            ray.getAppIcon(trashed_client.icon, self),
+            trashed_client.prettier_name())
+        act_x_trashed.setData(trashed_client.client_id)
         act_x_trashed.triggered.connect(self.showClientTrashDialog)
 
         self.ui.trashButton.setEnabled(
@@ -878,12 +960,12 @@ class MainWindow(QMainWindow):
             return
 
         for trashed_client in self._session.trashed_clients:
-            if trashed_client.data.client_id == client_id:
+            if trashed_client.client_id == client_id:
                 break
         else:
             return
 
-        dialog = child_dialogs.ClientTrashDialog(self, trashed_client.data)
+        dialog = child_dialogs.ClientTrashDialog(self, trashed_client)
         dialog.exec()
         if not dialog.result():
             return
@@ -951,6 +1033,17 @@ class MainWindow(QMainWindow):
             del self.script_action_dialog
             self.script_action_dialog = None
 
+    def makeAllDialogsReappear(self):
+        ok = QMessageBox.question(
+            self,
+            _translate('hidden_dialogs', 'Make reappear dialog windows'),
+            _translate('hidden_dialogs', 'Do you want to make reappear all dialogs you wanted to hide ?'))
+
+        if not ok:
+            return
+
+        RS.resetHiddens()
+
     def resizeWinWithMessages(self, messages_visible):
         if messages_visible:
             pass
@@ -997,3 +1090,15 @@ class MainWindow(QMainWindow):
     def enterEvent(self, event):
         self.mouse_is_inside = True
         QDialog.enterEvent(self, event)
+
+    def showEvent(self, event):
+        if CommandLineArgs.under_nsm:
+            if self._session._nsm_child is not None:
+                self._session._nsm_child.sendGuiState(True)
+        QMainWindow.showEvent(self, event)
+
+    def hideEvent(self, event):
+        if CommandLineArgs.under_nsm:
+            if self._session._nsm_child is not None:
+                self._session._nsm_child.sendGuiState(False)
+        QMainWindow.hideEvent(self, event)
