@@ -1,12 +1,15 @@
 from PyQt5.QtWidgets import (QLineEdit, QStackedWidget, QLabel, QToolButton,
                              QFrame)
-from PyQt5.QtGui import QFont, QFontDatabase, QFontMetrics, QPalette
+from PyQt5.QtGui import QFont, QFontDatabase, QFontMetrics, QPalette, QIcon
 from PyQt5.QtCore import Qt, QTimer, pyqtSignal
 
 import time
 
-class HideGuiButton(QToolButton):
-    toggleGui = pyqtSignal()
+from gui_tools import isDarkTheme
+from gui_signaler import Signaler
+
+class RayHackButton(QToolButton):
+    orderHackVisibility = pyqtSignal(bool)
 
     def __init__(self, parent):
         QToolButton.__init__(self, parent)
@@ -47,13 +50,14 @@ class HideGuiButton(QToolButton):
         self.setStyleSheet(style)
 
     def mousePressEvent(self, event):
-        self.toggleGui.emit()
+        self.orderHackVisibility.emit(not self.isChecked())
         # and not toggle button, the client will emit a gui state that will
         # toggle this button
 
 
 class OpenSessionFilterBar(QLineEdit):
     updownpressed = pyqtSignal(int)
+    key_event = pyqtSignal(object)
 
     def __init__(self, parent):
         QLineEdit.__init__(self)
@@ -61,6 +65,7 @@ class OpenSessionFilterBar(QLineEdit):
     def keyPressEvent(self, event):
         if event.key() in (Qt.Key_Up, Qt.Key_Down):
             self.updownpressed.emit(event.key())
+            self.key_event.emit(event)
         QLineEdit.keyPressEvent(self, event)
 
 
@@ -186,7 +191,7 @@ class StatusBar(QLineEdit):
             self.timer.stop()
 
     def setFontForText(self, text):
-        if QFontMetrics(self.ubuntu_font).width(text) > (self.width() - 10):
+        if QFontMetrics(self.ubuntu_font).width(text) >= (self.width() - 16):
             self.setFont(self.ubuntu_font_cond)
         else:
             self.setFont(self.ubuntu_font)
@@ -243,3 +248,58 @@ class StatusBar(QLineEdit):
 class StatusBarNegativ(StatusBar):
     def __init__(self, parent):
         StatusBar.__init__(self, parent)
+
+class FakeToolButton(QToolButton):
+    def __init__(self, parent):
+        QToolButton.__init__(self, parent)
+        self.setStyleSheet("QToolButton{border:none}")
+
+    def mousePressEvent(self, event):
+        self.parent().mousePressEvent(event)
+
+
+class favoriteToolButton(QToolButton):
+    def __init__(self, parent):
+        QToolButton.__init__(self, parent)
+        self.favorite_list = []
+        self.template_name = ""
+        self.template_icon = ""
+        self.factory = True
+        self.session = None
+
+        self.m_state = False
+        self.favicon_not = QIcon(':scalable/breeze/draw-star.svg')
+        self.favicon_yes = QIcon(':scalable/breeze/star-yellow.svg')
+        self.setIcon(self.favicon_not)
+
+    def setDarkTheme(self):
+        self.favicon_not = QIcon(':scalable/breeze-dark/draw-star.svg')
+        if not self.m_state:
+            self.setIcon(self.favicon_not)
+
+    def setSession(self, session):
+        self.session = session
+
+    def setTemplate(self, template_name: str,
+                    template_icon: str, factory: bool):
+        self.template_name = template_name
+        self.template_icon = template_icon
+        self.factory = factory
+
+    def setAsFavorite(self, bool_favorite: bool):
+        self.m_state = bool_favorite
+        if bool_favorite:
+            self.setIcon(self.favicon_yes)
+        else:
+            self.setIcon(self.favicon_not)
+
+    def mouseReleaseEvent(self, event):
+        QToolButton.mouseReleaseEvent(self, event)
+        if self.session is None:
+            return
+
+        if self.m_state:
+            self.session.removeFavorite(self.template_name, self.factory)
+        else:
+            self.session.addFavorite(self.template_name, self.template_icon,
+                                     self.factory)
